package com.tanksoftware.apps.copernicus;

import java.awt.*;
import javax.swing.*;
import com.juk.gui.*;

import java.io.*;
import java.util.List;
import java.util.HashMap;
import java.util.ArrayList;
import com.tanksoftware.io.*;
import com.tanksoftware.util.*;
import com.tanksoftware.protocol.OneTimeNOCList;
import java.awt.event.*;


import com.tanksoftware.protocol.MultiLingual;

/**
 * The main classes for the running of the application Copernicus.
 *
 * <p>Title: Copernicus</p>
 * <p>Description: Internet Password and Username tracking software</p>
 * <p>Copyright: Copyright (c) 2002 William Denniss</p>
 * <p>Company: Tank Software</p>
 * @author William Denniss
 * @version 1.4.0
 */

public class Copernicus extends JFrame {

	public static final String INI_FILE_NAME = "copernicus.ini";
	
	private SunFileFilter wxmlFilter;
	
	private final String pname = "copernicus";
	String lastLoadedFile = "";
	
	private int menulimit = 20;
	
	
	JPanel panel = new JPanel();
	JMenuBar mainMenuBar = new JMenuBar();
	JMenu menuFile = new JMenu();
	JMenu menuEdit = new JMenu();
	JMenu menuHelp = new JMenu();
	JMenuItem menuItemHelp = new JMenuItem();
	JMenuItem menuItemAbout = new JMenuItem();
	JMenuItem menuItemSave = new JMenuItem();
	
	JMenuItem menuNewSite = new JMenuItem();
	JMenuItem menuRenameCurrentSite = new JMenuItem();
	JMenuItem menuDeleteCurrentSite = new JMenuItem();
	JMenuItem menuNewLogin = new JMenuItem();
	JMenuItem menuRenameCurrentLogin = new JMenuItem();
	JMenuItem menuDeleteCurrentLogin = new JMenuItem();
	
	
	
	JMenu menuUsers = new JMenu();
	JMenu menuSites = new JMenu();
	JMenu menuLogins = new JMenu();
	XYLayout xYLayout1 = new XYLayout();
	JLabel welcomeWagonLabel = new JLabel();
	String welcomeUserLabel = "";
	
	JMenu menuLanguages;
	
	
	private String baseDirectory = "";
	//private TSFS fileSystem = new TSFS ();
	private List users = new ArrayList();
	private HashMap users_name = new HashMap();
	private HashMap users_filename = new HashMap();
	private Ini mainIni;
	private WXML currentUser;
	private WXMLSection currentSite;
	private WXMLSection currentLogin;
	private String currentUserFileName;
	private int lastUser = 0;
	
	private WXMLSection [] currentSortedSites;
	
	
	
	JButton editUsername = new JButton();
	JLabel siteNameLabel = new JLabel();
	JTextField siteNameText = new JTextField();
	JButton lookup = new JButton();
	JTextField loginNameText = new JTextField(11);
	JLabel loginLabel = new JLabel();
	JTextField usernameText = new JTextField(25);
	JLabel usernameLabel = new JLabel();
	JTextField passwordText = new JTextField(25);
	JLabel passwordLabel = new JLabel();
	JLabel commentLabel = new JLabel();
	JScrollPane commentTextScrollPane = new JScrollPane();
	JTextArea commentText = new JTextArea();
	JButton addSite = new JButton();
	JButton lookupLogin = new JButton();
	JButton exit = new JButton();
	JButton logout = new JButton();
	JButton save = new JButton();
	JButton saveLogin = new JButton();
	JButton deleteLogin = new JButton();
	JButton addLogin = new JButton();
	JButton deleteSite = new JButton();
	JCheckBox useEncryption = new JCheckBox();
	JMenuItem menuCreateNewUser = new JMenuItem();
	JMenuItem menuDeleteCurrentUser = new JMenuItem();
	JMenuItem menuLogout = new JMenuItem();
	JMenuItem menuExit = new JMenuItem();
	
	JButton next = new JButton (">");
	JButton back = new JButton ("<");
	
	private JPanel sitePanel;
	private JPanel loginPanel;
	
	private XYConstraints tabConsp1;
	private XYConstraints tabConsp2;
	private XYConstraints editUsernameXYConstraints;
	
	
	
	private int searchNum = 0;
	//private int searchNumLogin = 0;
	
	private boolean saveNeeded = false;
	
	public static final String PROGRAM_NAME = "Copernicus";
	public static final String VERSION_NUMBER = "1.4.0";
	public static final String DATE_CREATED = "05 April 2002";
	public static final String DATE_UPDATED = "17 August 2002";
	public static final String AUTHOR = "William Denniss";
	public static WillTank willTankPanel = new WillTank(PROGRAM_NAME, VERSION_NUMBER, DATE_UPDATED);
	JMenuItem menuSetAsDefault = new JMenuItem();
	JMenuItem menuSaveAs = new JMenuItem();
	JMenuItem menuAddUserFile = new JMenuItem();
	JMenuItem menuChangeName = new JMenuItem();
	JTabbedPane jTabbedPane1 = new JTabbedPane();
	JTabbedPane jTabbedPane2 = new JTabbedPane();
	String currentUserFile = "";

	public Copernicus()  {
		setTitle("Copernicus 1.4");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	

		setIconImage(new ImageIcon("data/Copernicus/copernicus.gif").getImage());
		setBounds(50,50,455,530);
		setLocation(50,50);

		
		// Sets up the file filter
		wxmlFilter = new SunFileFilter();
		wxmlFilter.addExtension("wxml");
		wxmlFilter.addExtension("xml");
		wxmlFilter.addExtension("usr");
		wxmlFilter.setDescription("Copernicus User Files");
		
		// Loads the multilingual phrase lookup table.
		/*try {
		    MultiLingual.loadStringTable();
		} catch (IOException e) {
		    System.out.println("Language file: data/language.tdt not found.  This program will now run only in English.");
		}*/
		
		MultiLingual.loadProgramPack("copernicus", "GBR", "data/lanpacks/");
		
		// Starts the about dialog
		willTankPanel.startDialog();
		
		// Initialises to English.
		updateLanguageTexts("GBR");
		
		// Listen for the window shutdown.
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				exit_actionPerformed(null);
			}
		});
		
		
		/*
		 * Button anonymouse inner classes.
		 * Also, some buttons have Accelerators, Mnemomics and Icons.
		 */
		 
		menuNewSite.addActionListener(new java.awt.event.ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			checkLoginChanged(true);
			addSite_actionPerformed(e);
		
		    }
		});
		
		
		back.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				navigateLogin(-1);
			}
		});
		
		next.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				navigateLogin(1);
			}
		});
		
		menuRenameCurrentSite.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
		menuRenameCurrentSite.setMnemonic(KeyEvent.VK_R);
		
		menuRenameCurrentSite.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				menuRenameCurrentSite_actionPerformed(e);
			}
		});
		menuDeleteCurrentSite.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				deleteSite_actionPerformed(e);
			}
		});
		menuNewLogin.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addLogin_actionPerformed(e);
			}
		});
		menuRenameCurrentLogin.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				menuRenameCurrentLogin_actionPerformed(e);
			}
		});
		menuDeleteCurrentLogin.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				deleteLogin_actionPerformed(e);
			}
		});
		
		
		menuExit.setMnemonic(KeyEvent.VK_X);
		menuHelp.setMnemonic(KeyEvent.VK_H);
		menuItemHelp.setMnemonic(KeyEvent.VK_E);
		
		menuItemHelp.setIcon(new ImageIcon ("data/images/help.png"));
		
		menuItemHelp.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showHelp();
			}
		});
		
		menuItemAbout.setIcon(new ImageIcon ("data/images/tsicon.png"));
		menuItemAbout.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				willTankPanel.showDialog();
			}
		});
		
		menuCreateNewUser.setIcon(new ImageIcon ("data/images/new.png"));
		
		menuDeleteCurrentUser.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				deleteCurrentUser();
			}
		});
		
		menuLogout.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				logout();
			}
		});
		
		menuExit.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exit_actionPerformed(e);
			}
		});
		
		menuSetAsDefault.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setCurrentAsDefault();
			}
		});
		
		
		menuItemSave.setIcon(new ImageIcon ("data/images/save.png"));
		menuItemSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		menuItemSave.setMnemonic(KeyEvent.VK_S);
		
		menuItemSave.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				save_actionPerformed(e);
			}
		});
		
		
		menuSaveAs.setIcon(new ImageIcon ("data/images/blank.png"));
		menuSaveAs.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveUserFileAs_actionPerformed(e);
			}
		});
		
		
		
		menuAddUserFile.setIcon(new ImageIcon ("data/images/open.png"));
		menuAddUserFile.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addUserFile();
			}
		});
		
		
		menuChangeName.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editUsername_actionPerformed(null);
			}
		});
		
		
		
		editUsername.setIcon(new ImageIcon ("data/images/edit.png"));
		editUsername.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				editUsername_actionPerformed(e);
			}
		});
		siteNameText.setText("www.omegadelta.net");
		lookup.setIcon(new ImageIcon ("data/images/lookup.png"));
		lookup.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				checkLoginChanged(true);
				lookupSite(true, false);
			}
		});
		addSite.setIcon(new ImageIcon ("data/images/new.png"));
		addSite.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				checkLoginChanged(true);
				addSite_actionPerformed(e);
			}
		});
		
		
		
		lookupLogin.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				checkLoginChanged(true);
				lookupLogin(loginNameText.getText());
			}
		});
		lookupLogin.setIcon(new ImageIcon ("data/images/lookup.png"));
		exit.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exit_actionPerformed(e);
			}
		});
		logout.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				logout();
			}
		});
		save.setIcon(new ImageIcon ("data/images/save.png"));
		save.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				save_actionPerformed(e);
			}
		});
		saveLogin.setIcon(new ImageIcon ("data/images/save.png"));
		saveLogin.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveLogin_actionPerformed(e);
			}
		});
		
		addLogin.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addLogin_actionPerformed(e);
			}
		});
		
		deleteLogin.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				deleteLogin_actionPerformed(e);
			}
		});
		
		
		deleteSite.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				deleteSite_actionPerformed(e);
			}
		});
		
		menuCreateNewUser.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				newUser();
			}
		});
		
		useEncryption.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				useEncryption_actionPerformed(e);
			}
		});
		
		passwordText.addKeyListener(new java.awt.event.KeyListener() {
			public void keyPressed(KeyEvent e) {
			}
			public void keyReleased(KeyEvent e) {
				keypressed_action(e);
			}
			public void keyTyped(KeyEvent e) {
			}
		});
		usernameText.addKeyListener(new java.awt.event.KeyListener() {
			public void keyPressed(KeyEvent e) {
			}
			public void keyReleased(KeyEvent e) {
				keypressed_action(e);
			}
			public void keyTyped(KeyEvent e) {
			}
		});
		commentText.addKeyListener(new java.awt.event.KeyListener() {
			public void keyPressed(KeyEvent e) {
			}
			public void keyReleased(KeyEvent e) {
				keypressed_action(e);
			}
			public void keyTyped(KeyEvent e) {
			}
		});
		
		siteNameText.addKeyListener(new java.awt.event.KeyListener() {
			public void keyPressed(KeyEvent e) {
			}
			public void keyReleased(KeyEvent e) {
				siteNameText_keyPressed(e);
			}
			public void keyTyped(KeyEvent e) {
			}
		});
		
		
		/*
		 *
		 * Set up the main GUI frames and panels.
		 *
		 */
		
		panel.setLayout(xYLayout1);
		
		// Welcome Labels
		panel.add(welcomeWagonLabel,  new XYConstraints(5, 5, -1, 29));
		editUsernameXYConstraints = new XYConstraints(80, 35, -1, -1);
		panel.add(editUsername, editUsernameXYConstraints);
		
		//Lower button panel
		JPanel buttonPanel = new JPanel ();
		buttonPanel.add(useEncryption);
		buttonPanel.add(save);
		buttonPanel.add(logout);
		buttonPanel.add(exit);
		panel.add(buttonPanel, new XYConstraints(21, 447, -1, -1));
			
		//Site Panel
		sitePanel = new JPanel ();
		sitePanel.setLayout(new XYLayout ());
		sitePanel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED));
		sitePanel.add(siteNameLabel, new XYConstraints(34-19, 61-57, -1, 21));
		sitePanel.add(siteNameText, new XYConstraints(31-19, 84-57, 232, 24));     
		sitePanel.add(lookup, new XYConstraints(270-19, 83-57, -1, -1));
		sitePanel.add(addSite,   new XYConstraints(220-19, 113-57, -1, 23));
		tabConsp1 = new XYConstraints(19, 65, -1, 92);
		panel.add(sitePanel, tabConsp1);
		
		//Login Panel
		loginPanel = new JPanel();
		loginPanel.setLayout(new XYLayout ());
		loginPanel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED));
		
		JPanel loginTextButtons = new JPanel();
		loginTextButtons.add(loginLabel);
		loginTextButtons.add(loginNameText);
		loginTextButtons.add(lookupLogin);
		loginPanel.add(loginTextButtons, new XYConstraints(37-13, 175-165, -1, -1));
		
		JPanel loginNavigationButtons = new JPanel ();
		loginNavigationButtons.add(saveLogin);
		loginNavigationButtons.add(new JLabel("   "));
		loginNavigationButtons.add(back);
		loginNavigationButtons.add(next);
		loginNavigationButtons.add(new JLabel("   "));
		loginNavigationButtons.add(addLogin);
		loginPanel.add(loginNavigationButtons,  new XYConstraints(36-13, 203-165+5, -1, -1));
		
		JPanel usernamePanel = new JPanel ();
		usernamePanel.add(usernameLabel);
		loginNavigationButtons.add(new JLabel("  "));
		usernamePanel.add(usernameText);
		loginPanel.add(usernamePanel, new XYConstraints(36-13, 239-165, -1, -1));
		
		
		JPanel passwordPanel = new JPanel();
		passwordPanel.add(passwordLabel);
		loginNavigationButtons.add(new JLabel("  "));
		passwordPanel.add(passwordText);
		loginPanel.add(passwordPanel, new XYConstraints(36-13, 267-165, -1, -1));
		
		
		
		loginPanel.add(commentTextScrollPane, new XYConstraints(22-13, 319-165, 390, 98));
		loginPanel.add(commentLabel,  new XYConstraints(22-13, 297-165, 138, 21));
		
		
		tabConsp2 = new XYConstraints(13, 175, 416, 262);
		panel.add(loginPanel, tabConsp2);
		
		
		/*
		 * Sets up the menu.
		 *
		 */
		menuFile.add(this.menuCreateNewUser);
		
		this.setJMenuBar(mainMenuBar);
		welcomeWagonLabel.setFont(new java.awt.Font("Dialog", 1, 13));
		
		menuFile.setMnemonic(KeyEvent.VK_F);
		menuEdit.setMnemonic(KeyEvent.VK_E);
		this.getContentPane().add(panel, BorderLayout.CENTER);
		
		mainMenuBar.add(menuFile);
		mainMenuBar.add(menuEdit);
		mainMenuBar.add(menuUsers);
		mainMenuBar.add(menuSites);
		mainMenuBar.add(menuLogins);
		mainMenuBar.add(Box.createHorizontalGlue());
		
		menuLanguages = MultiLingual.constructMenu(pname);
		if (menuLanguages != null)
			mainMenuBar.add(menuLanguages);
		mainMenuBar.add(menuHelp);
		
		/*
		 * Languages
		 */
		if (menuLanguages != null) {
			for (int i = 0; i < menuLanguages.getItemCount(); i++) {
				JMenuItem item = menuLanguages.getItem(i);
				item.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						changeLanguage(((JMenuItem)e.getSource()).getText());
					}
				});
			
			}
		}
		/*
		menuLanguages.add(menuLanguagesItems[lanOpt]);
		menuLanguagesItems[lanOpt].setIcon(new ImageIcon ("data/images/flags/chin.gif"));
		menuLanguagesItems[lanOpt].addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeLanguage(MultiLingual.CHINESE);
			}
		});
		lanOpt++
		*/
		
		
		menuHelp.add(menuItemHelp);
		menuHelp.addSeparator();
		menuHelp.add(menuItemAbout);

		 
		commentTextScrollPane.getViewport().add(commentText, null);
		menuFile.add(menuCreateNewUser);
		menuFile.add(menuAddUserFile);
		menuFile.add(menuItemSave);
		menuFile.add(menuSaveAs);
		menuFile.addSeparator();
		menuFile.add(menuChangeName);
		menuFile.add(menuSetAsDefault);
		menuFile.addSeparator();
		menuFile.add(menuDeleteCurrentUser);
		menuFile.addSeparator();
		menuFile.add(menuLogout);
		menuFile.add(menuExit);
		
		
		menuEdit.add(menuNewSite);
		menuEdit.add(menuRenameCurrentSite);
		menuEdit.add(menuDeleteCurrentSite);
		menuEdit.addSeparator();
		menuEdit.add(menuNewLogin);
		menuEdit.add(menuRenameCurrentLogin);
		menuEdit.add(menuDeleteCurrentLogin);
		
		
		// Initialise
		logout();
		
		
		// Gets the program path.
		File f = new File("");
		baseDirectory = f.getAbsolutePath() + "/";
		
		
		/*
		 *
		 * Loads the INI settings.
		 *
		 */
		 
		try {
			mainIni = new Ini (baseDirectory + Copernicus.INI_FILE_NAME, null);
		} catch (IOException e) {
			try {
			    TSFS.TSFSFile newIni = new TSFS.TSFSFile(baseDirectory + "copernicus.ini", Copernicus.firstSettings(baseDirectory));
			    TSFS.TSFSFile newUser = new TSFS.TSFSFile(baseDirectory + "user1.wxml", Copernicus.firstUser(""));
			    newIni.writeFile();
			    newUser.writeFile();
			    mainIni = new Ini (baseDirectory + "copernicus.ini", null);
			} catch (IOException e2) {
			}
		}
		
		
		try {
			Ini.Section mainsettings = (Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS");

			// Loads the language
			try {
			
				changeLanguage(mainsettings.getSubSectionValue("language"));
			} catch (Exception e) {
				changeLanguage("GBR");
			}
			
			// Loads the users, note: 1000 user max.
			for (int i = 0; i < 1000; i++) {
			    try {
				users.add(mainsettings.getSubSectionValue("user" + i));
				lastUser = i;
				String userfilenamestr = mainsettings.getSubSectionValue("user" + i);
				users_name.put(mainsettings.getSubSectionValue("user_" + userfilenamestr+ "_name"), userfilenamestr );
				users_filename.put(userfilenamestr ,mainsettings.getSubSectionValue("user_" + userfilenamestr+ "_name"));
			    } catch (Ini.SubSectionNotFoundException e ) {
			    }
			}
			
			// Loads the menu limit for the site menu (default 20)
			try {
				menulimit = Integer.parseInt(mainsettings.getSubSectionValue("menulimit"));
			} catch (Exception e ) {
			}
		
			
			
			populateUsersList();
		
			//try to load the default user
			try {
			    String firstUser = mainsettings.getSubSectionValue("startingUser");
			    if (!firstUser.equals(""))
				loadUser(firstUser);
			} catch (Exception e ) {
				showWelcome();
			}
			
		} catch (Ini.SubSectionNotFoundException e) {
		}
		
		lastLoadedFile = baseDirectory;
		setVisible(true);

	
	}
    
    
	/**
	 * Runs Copernicus.  Currently command line arguments are ignored
	 *
	 *@param args Any command line arguments.
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
		} catch (Exception e) {
			System.out.println("Failed to load KunststoffLookAndFeel UI.");
		}
		Copernicus copernicus = new Copernicus();
	}

	/**
	 * Shows/updates the welcome notice
	 */
	private void showWelcome () {
		try {
			welcomeUserLabel = currentUser.getSubSection("dpass").getAttributeValue("username");
		} catch (NullPointerException e) {
			welcomeUserLabel = "please select a user";
		}
		welcomeWagonLabel.setText(MultiLingual.translateWord(pname, "Welcome to Copernicus") + ", " + MultiLingual.translateWord(pname, welcomeUserLabel) + ".");
	}

	/**
	 * Checks to see if the file needs saving.  If it does, the user is asked if they would
	 * like to save or not.  If they would, the file is then saved.
	 *
	 */
	private void checkSave () {
		if (this.save.isEnabled() && currentUser != null && saveNeeded) {
			int result = JOptionPane.showConfirmDialog(null, MultiLingual.translateWord(pname, "Do you want to save the current user first") + "?", "Save Current User?", JOptionPane.YES_NO_OPTION);
			if (result == JOptionPane.YES_OPTION) {
				this.save_actionPerformed(null);
			}
		}
	}

	/**
	 * Logs off a user.
	 *
	 */
	private void logout () {
		checkLoginChanged(true);
		checkSave();
		
		this.menuSetAsDefault.setEnabled(false);
		this.menuSaveAs.setEnabled(false);
		this.menuChangeName.setEnabled(false);
		this.menuDeleteCurrentUser.setEnabled(false);
		this.menuLogout.setEnabled(false);
		
		this.siteNameText.setEnabled(false);
		this.usernameText.setEnabled(false);
		this.loginNameText.setEnabled(false);
		this.commentText.setEnabled(false);
		this.lookup.setEnabled(false);
		this.save.setEnabled(false);
		this.menuItemSave.setEnabled(false);
		this.addSite.setEnabled(false);
		this.menuNewSite.setEnabled(false);
		this.saveLogin.setEnabled(false);
		
		this.deleteLogin.setEnabled(false);
		
		this.menuRenameCurrentLogin.setEnabled(false);
		this.menuDeleteCurrentLogin.setEnabled(false);
		this.addLogin.setEnabled(false);
		this.menuNewLogin.setEnabled(false);	
		this.saveLogin.setEnabled(false);
		this.lookupLogin.setEnabled(false);
		this.next.setEnabled(false);
		this.back.setEnabled(false);
		
		
		this.passwordText.setEnabled(false);
		this.deleteSite.setEnabled(false);
		this.menuRenameCurrentSite.setEnabled(false);
		this.menuDeleteCurrentSite.setEnabled(false);
		this.useEncryption.setEnabled(false);
		this.menuLogins.setEnabled(false);
		this.menuSites.setEnabled(false);
		this.logout.setEnabled(false);
		this.editUsername.setVisible(false);
		this.siteNameText.setText("www.omegadelta.net");
		this.loginNameText.setText("");
		this.usernameText.setText("");
		this.passwordText.setText("");
		this.commentText.setText("");
		
		currentUser = null;
		currentSite = null;
		currentLogin = null;
		currentUserFileName = null;
		
		
		this.menuLogins.removeAll();
		this.menuSites.removeAll();
		
		saveNeeded = false;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);
		
		showWelcome();
	}
	
	/**
	 * Loads a given user.  If the load "fails" without encryption, then it is retried with encryption.
	 *
	 *@param filename the user file to load.
	 */
	private void loadUser (String filename) {
		this.welcomeWagonLabel.setText("Loading file, please wait.");
		
		// Logs out the current user.
		logout();
		
		System.out.println("Loading user file: " + filename);
		repaint();
		
		/*
		* Read file contents
		*/
		TSFS.TSFSFile currentFile = null;
		try {
			currentFile = new TSFS.TSFSFile(filename);
		
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(null, MultiLingual.translateWord(pname, "Error Reading File") + ": " + filename + ".  " + MultiLingual.translateWord(pname, "Probable bad file (missing or unavailible)") + ".", "Reading Error.", JOptionPane.OK_OPTION);
			logout();
		}
		
		if (currentFile != null) {
			String fileContents = currentFile.getFileContents();
		
			try {
			
				//Try to load the file without encrypting
				currentUser = WXML.parseString(fileContents);
				WXMLSection test = currentUser.getSubSection("dpass");
				this.useEncryption.setSelected(false);
				if (currentUser == null || test == null)
					throw new NullPointerException ();
			
			} catch (Exception e) {
		
		
				this.welcomeWagonLabel.setText("Waiting for password...");
				//that failed, so now try to decrypt it
				try {
					String password = JOptionPane.showInputDialog(MultiLingual.translateWord(pname, "The file") + " " + filename + " " + MultiLingual.translateWord(pname, "appears to be encrypted.  Please enter a password."));
					this.welcomeWagonLabel.setText("Authenticating user...");
					
					//Generate key from password
					String key = OneTimeNOCList.generateRandomKey(fileContents, password, 0);
					
					//decrypt
					BufferedReader brKey = new BufferedReader(new StringReader(key));
					BufferedReader brEncrypted = new BufferedReader(new StringReader(fileContents));
					StringWriter plainText = new StringWriter ();
					BufferedWriter bwPlain = new BufferedWriter(plainText);
			
					try {
						OneTimeNOCList.decrypt(bwPlain, brEncrypted, brKey);
						this.useEncryption.setSelected(true);
					} catch (IOException ex) {
					}
					
					try {
						currentUser = WXML.parseString(plainText.toString());
						WXMLSection test = currentUser.getSubSection("dpass");
						if (test == null)
						throw new NullPointerException ();
					} catch (Exception ex) {
						this.welcomeWagonLabel.setText("Incorrect Password.");
						
						JOptionPane.showMessageDialog(null, MultiLingual.translateWord(pname, "Error Reading File.  Possible Incorrect Password or invalid file."), "Reading Error.", JOptionPane.OK_OPTION);
						throw new NullPointerException ();
					}
					
					//try to decrypt
				} catch (NullPointerException exp) {
					
					// Bad password or bad file.  Bad file could be ill-formatted or
					// corrupted, or non-existant.
					logout();
					currentUser = null;
				}
			}
		
			currentUserFileName = filename;
		
		} else {
			currentUser = null;	
		}
		
		if (currentUser != null) {
			this.siteNameText.setEnabled(true);
			this.lookup.setEnabled(true);
			this.addSite.setEnabled(true);
			this.menuNewSite.setEnabled(true);
			//this.save.setEnabled(true);
			this.useEncryption.setEnabled(true);
			this.menuLogins.setEnabled(true);
			this.menuSites.setEnabled(true);
			this.logout.setEnabled(true);
			populateSiteList();
			this.menuSetAsDefault.setEnabled(true);
			this.menuSaveAs.setEnabled(true);
			this.menuChangeName.setEnabled(true);
			this.menuDeleteCurrentUser.setEnabled(true);
			this.menuLogout.setEnabled(true);
			showWelcome();
			if (currentUser.getSubSection("dpass").getAttributeValue("username").equals("New User")) {
				this.editUsername.setVisible(true);
			} else {
				this.editUsername.setVisible(false);
			}
			this.currentUserFile = filename;
			
			System.out.println("User loaded");
			this.siteNameLabel.setText("No site yet loaded");
			this.siteNameText.setText("");
			
			try {
				users_name.put(currentUser.getSubSection("dpass").getAttributeValue("username"), filename);
				users_filename.put(filename, currentUser.getSubSection("dpass").getAttributeValue("username"));
				((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).setSubSectionValue("user_" + filename +"_name",  currentUser.getSubSection("dpass").getAttributeValue("username"));
				saveIni();
			} catch (Exception e) {
			}
			
			populateUsersList();
		} else {
			System.out.println("Failed to load user.");
		
		}
	}

	
	/**
	 * Performs the site search.  If an exact match is found, that is loaded, or the first partial match.
	 * if a partial match was already found, the next one is loaded.
	 *
	 *@param createNew If true, will create a new site when it isn't matched.
	 *@param startFromStart If true, will start the match search from the beginning of the list.
	 */
	private void lookupSite (boolean createNew, boolean startFromStart) {

		WXMLSection [] sites =  currentSortedSites;
		
		currentSite = null;
		
		boolean exactMatch = false;

		//check for exact match
		for (int i = 0; i < sites.length; i++) {
			if (sites[i].getAttributeValue("name").equalsIgnoreCase(siteNameText.getText())) {
				currentSite = sites[i];
				searchNum = 0;
				lookup.setText(MultiLingual.translateWord(pname, "Lookup"));
				exactMatch = true;
				break;
			}
		}
		
		
		//no exact match, so check for partial match
		if (currentSite == null) {	
			if (startFromStart)
				searchNum = 0;
			for (int i = searchNum; i < sites.length; i++) {
				if (sites[i].getAttributeValue("name").toLowerCase().indexOf(siteNameText.getText().toLowerCase()) != -1) {
					currentSite = sites[i];
					searchNum = i+1;
					lookup.setText(MultiLingual.translateWord(pname, "Find Next"));
					
					break;
				}
			}
		}
		
		
		
		if (currentSite == null) {
			searchNum = 0;
			
			//site does not exits create new one?
			if (createNew) {
				int result = JOptionPane.showConfirmDialog(null, MultiLingual.translateWord(pname, "No more matches found, do you want to create a new site named") + " " + siteNameText.getText() + "?", "Create new site?", JOptionPane.YES_NO_OPTION);
				
				if (result == JOptionPane.YES_OPTION) {
					createNewSite (siteNameText.getText());
					
					this.siteNameLabel.setText("Site: " + this.siteNameText.getText());
					this.usernameText.setText("");
					this.passwordText.setText("");
					this.commentText.setText("");
					this.loginNameText.setText("Main Login");
				}
			}
		} else {
			if (exactMatch && !startFromStart) {
				this.siteNameLabel.setText("Site: " + currentSite.getAttributeValue("name") + "  (exact match)");
			} else if (!exactMatch && !startFromStart ) {
				this.siteNameLabel.setText("Site: " + currentSite.getAttributeValue("name") + "  (partial match)");			
			} else {
				this.siteNameLabel.setText("Site: " + currentSite.getAttributeValue("name"));
			}
			
			WXMLSection [] logins = currentSite.getSubSectionGroup("login");
			try {
				this.loginNameText.setText(logins[0].getAttributeValue("userid"));
				lookupLogin(logins[0].getAttributeValue("userid"));
			} catch (ArrayIndexOutOfBoundsException e) {
				lookupLogin ("Main Login");
				saveNeeded = true;
				menuItemSave.setEnabled(saveNeeded);
				save.setEnabled(saveNeeded);
				deleteLogin.setEnabled(false);
				
				menuDeleteCurrentLogin.setEnabled(false);
				menuRenameCurrentLogin.setEnabled(false);
				
			}
		}
		
		try {
			populateLoginList();
		} catch (Exception e) {
		}
		
			this.usernameText.setEnabled(true);
			this.loginNameText.setEnabled(true);
			this.commentText.setEnabled(true);
			this.passwordText.setEnabled(true);
		
		if (currentSite == null) {
			this.addSite.setEnabled(true);
			this.menuNewSite.setEnabled(true);
		} else {
			this.deleteSite.setEnabled(true);
			this.menuRenameCurrentSite.setEnabled(true);
			this.menuDeleteCurrentSite.setEnabled(true);
			this.lookupLogin.setEnabled(true);
			this.next.setEnabled(true);
			this.back.setEnabled(true);
		}
	}
	
	/**
	 * Used to lookup an existing login, or create a new one.
	 * The logins are searched for a match and if there is, it is loaded,
	 * if there isn't a new one is created ready for use.
	 *
	 *@param login the login to search for.
	 */
	private void lookupLogin (String login) {
		
		WXMLSection [] logins;
		try {
		    logins =  currentSite.getSubSectionGroup("login");
		} catch (NullPointerException e) {
		    return;
		}
		loginNameText.setText(login);
		
		currentLogin = null;
		for (int i = 0; i < logins.length; i++) {
		    //must match exactally
			if (logins[i].getAttributeValue("userid").equalsIgnoreCase(this.loginNameText.getText())) {
				currentLogin = logins[i];
				break;
			}
		}
		
		if (currentLogin == null) {
		
			this.usernameText.setText("");
			this.passwordText.setText("");
			this.commentText.setText("");
			this.loginNameText.setText(this.loginNameText.getText());
			saveLogin.setEnabled(true);
		
			saveNeeded = true;
			menuItemSave.setEnabled(saveNeeded);
			save.setEnabled(saveNeeded);
			
		} else {
		    this.usernameText.setText(currentLogin.getAttributeValue("username"));
		    this.passwordText.setText(currentLogin.getAttributeValue("pass"));
		    this.commentText.setText(currentLogin.getAttributeValue("comment"));
		    this.loginNameText.setText(currentLogin.getAttributeValue("userid"));
		    this.deleteLogin.setEnabled(true);
		    
		    this.menuDeleteCurrentLogin.setEnabled(true);
		    this.menuRenameCurrentLogin.setEnabled(true);
		    this.addLogin.setEnabled(true);
		    this.menuNewLogin.setEnabled(true);
		    this.saveLogin.setEnabled(false);
		}
	}
	
	/**
	 * Returns the default ini file for the given directory.
	 *
	 *@param baseDirectory The program directory.
	 *@return the default ini file for the given directory.
	 */
	public static String firstSettings (String baseDirectory) {
	
		return
			"[MAIN_SETTINGS]\n" +
			"user1=" + baseDirectory + "user1.wxml\n" +
			"startingUser=" + baseDirectory + "user1.wxml\n" +
			"language=0\n" +
			"menulimit=20";
	}

	public static String firstUser (String number) {
	
		return
			"<dpass username=\"New User "+ number + "\">\n" +
			"\t<site>\n" +
			"\t<name>www.omegadelta.net</name>\n" +
			"\t\t<login userid=main username=user1 pass=xyz>\n" +
			"\t\t\t<comment>Main Account</comment>\n" +
			"\t\t</login>\n" +
			"\t\t<login userid=second username=user2 pass=xyz>\n" +
			"\t\t\t<comment>Secondary Account</comment>\n" +
			"\t\t</login>\n" +
			"\t</site>\n" +
			"</dpass>\n";
	}
	
	
	/**
	 * Populates the user menu list.
	 *
	 */
	private void populateUsersList () {
		this.menuUsers.removeAll();
		for (int i = 0; i < users.size(); i++) {
			
			/*
			 * Display only those files which can be read.
			 */
		    try {
			String filename = (String) users.get(i);
			new BufferedReader(new FileReader(filename));
			
				String username = (String) users_filename.get((String) users.get(i));
				if (username == null)
					username = (String) users.get(i);
			    JMenuItem newItem = new JMenuItem(username);
			    this.menuUsers.add(newItem);
			    newItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
				    String menutext = ((JMenuItem) e.getSource()).getText();
				    String filename = (String) users_name.get(menutext);
				    if (filename == null)
					    filename = menutext;
				    loadUser(filename);
				}
			    });
		    } catch (IOException e) {
			//ignore user.
		    }
		}
	}


	/**
	 * Populates the menu site list alphabetically, and limiting the number of sites listed per
	 * sub menu to 20 (or the number read in from the ini).
	 *
	 */
	private void populateSiteList () {
		WXMLSection [] sites =  currentUser.getSubSection("dpass").getSubSectionGroup("site");
		
		/*
		 * O(n^2) Selection Sort
		 */
		for (int i = 0; i < sites.length; i++) {
			for (int k = i+1; k < sites.length; k++) {
				if (((String) sites[k].getAttributeValue("name")).compareToIgnoreCase((String) sites[i].getAttributeValue("name")) < 0 ) {
					WXMLSection tmp = sites[i];
					sites[i] = sites[k];
					sites[k] = tmp;
				} else {
				}
			}
		}
		
		currentSortedSites = sites;
		
		int numberSubMenus = (int) ((double) (sites.length-1) / (double) menulimit) +1;
		
		
		JMenu submenus [] = new JMenu [numberSubMenus];
		for (int i = 0; i < submenus.length; i++) {
			submenus[i] = new JMenu ("From to To");
		}
		int currentCount = 0;
		
		this.menuSites.removeAll();
	
		for (int i = 0; i < sites.length; i++) {
		    JMenuItem newItem = new JMenuItem((String) sites[i].getAttributeValue("name"));
	
			if (numberSubMenus == 1) {
			    this.menuSites.add(newItem);
			} else {
				submenus[ (int) ((double) (i) / (double) menulimit)].add(newItem);
			}
		   
	
		    newItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
			    checkLoginChanged(true);
			    siteNameText.setText(((JMenuItem) e.getSource()).getText());
			    lookupSite(false, true);
			}
		    });
	
		}
		
		if (numberSubMenus != 1) {
			for (int i = 0; i < submenus.length; i++) {
				menuSites.add (submenus[i]);
				int num = submenus[i].getItemCount();
				String from = "";
				try {
					from = submenus[i].getItem(0).getText().substring(0,3);
				} catch (StringIndexOutOfBoundsException e) {
					from = submenus[i].getItem(0).getText();
				}
				String to = "";
				try {
					to = submenus[i].getItem(num-1).getText().substring(0,3);
				} catch (StringIndexOutOfBoundsException e) {
					to = submenus[i].getItem(num-1).getText();
				}
	
				
				submenus[i].setText(from + " to " + to);
			}
		}
	}

	
	/**
	 * Returns a list of all of the logins, listed alphabetically.
	 *
	 *@return a list of all of the logins, listed alphabetically.
	 */
	private WXMLSection [] getOrdedLogins () {
		WXMLSection [] logins =  currentSite.getSubSectionGroup("login");
	    	
		/*
		 * O(n^2) Selection Sort
		 */
		for (int i = 0; i < logins.length; i++) {
			for (int k = i+1; k < logins.length; k++) {
				if (((String) logins[k].getAttributeValue("userid")).compareTo((String) logins[i].getAttributeValue("userid")) < 0 ) {
					WXMLSection tmp = logins[i];
					logins[i] = logins[k];
					logins[k] = tmp;
				} else {
				}
			}
		}
			
	    
	    return logins;	    
	}

	/**
	 * Populate the Login menu with an orded list of logins.
	 */
	private void populateLoginList () {
		WXMLSection [] logins;
		this.menuLogins.removeAll();
		try {
			logins = getOrdedLogins();
		} catch (NullPointerException e) {
			return;
		}
		for (int i = 0; i < logins.length; i++) {
			JMenuItem newItem = new JMenuItem((String) logins[i].getAttributeValue("userid"));
			this.menuLogins.add(newItem);
			newItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					checkLoginChanged(true);
					lookupLogin(((JMenuItem) e.getSource()).getText());
				}
			});

		}
	}
	
	/**
	 * Allows the user to change the Copernicus display name of the user
	 *
	 *@param e The ActionEvent.
	 */
	private void editUsername_actionPerformed(ActionEvent e) {
		String newUsername = JOptionPane.showInputDialog(MultiLingual.translateWord(pname, "Please enter a new Username") + ".");
		
		if (newUsername != "" && newUsername != null) {
			currentUser.getSubSection("dpass").setAttributeValue("username", newUsername);
			this.editUsername.setVisible(false);
			saveNeeded = true;
			showWelcome();
			menuItemSave.setEnabled(saveNeeded);
			save.setEnabled(saveNeeded);

			try {
				users_name.put(currentUser.getSubSection("dpass").getAttributeValue("username"), currentUserFileName);
				users_filename.put(currentUserFileName, currentUser.getSubSection("dpass").getAttributeValue("username"));
				((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).setSubSectionValue("user_" + currentUserFileName +"_name",  currentUser.getSubSection("dpass").getAttributeValue("username"));
				saveIni();
				populateUsersList();
			} catch (Exception eb) {
			}
		}
		
	
	}
	
	/**
	 * Exit the program nicly.
	 *
	 *@param e The ActionEvent.
	 */
	private void exit_actionPerformed(ActionEvent e) {
		checkLoginChanged(true);
		checkSave();
		System.out.println("program finished - exit");
		System.exit(0);
	}
	
	/**
	 * Check to see if the login was changed.
	 *
	 *@param showConfirm Specifies if a dialog box will be shown to the user should the login be changed.
	 *@return Whether or not the login was changed.
	 */
	private boolean checkLoginChanged (boolean showConfirm) {
		
		try {
			boolean loginChanged = false;
			
			if (!usernameText.getText().equals(this.currentLogin.getAttributeValue("username")))
				loginChanged = true;
			if (!passwordText.getText().equals(this.currentLogin.getAttributeValue("pass")))
				loginChanged = true;
			if (!commentText.getText().equals(this.currentLogin.getAttributeValue("comment")))
				loginChanged = true;
			
			if (loginNameText.getText().equals(""))
				loginChanged = false;
			
			
			if (loginChanged && showConfirm) {
				int result = JOptionPane.showConfirmDialog(null, MultiLingual.translateWord(pname, "Do you want to apply changes to the current login first?"), "Save Current Login?", JOptionPane.YES_NO_OPTION);
				if (result == JOptionPane.YES_OPTION) {
					saveLogin_actionPerformed(null);
				}
			
			}
			
			
			if (loginChanged) {
				saveLogin.setEnabled(true);
				
				saveNeeded = true;
				menuItemSave.setEnabled(saveNeeded);
				save.setEnabled(saveNeeded);
			
			} else {
				saveLogin.setEnabled(false);    
			}
			
			return loginChanged;
		} catch (NullPointerException e) {
			try {
				if (currentLogin == null && !loginNameText.getText().equals(""))
					return true;
				else
					return false;
			
			} catch (NullPointerException e2) {
				return false;
			}
		}
	
	
	}

	/**
	 * Updates the use encryption state.
	 */
	private void useEncryption_actionPerformed (ActionEvent evt) {
		saveNeeded = true;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);	    
	}
	
	
	/**
	 * Saves the current user to disk.
	 *
	 *@param e The ActionEvent.
	 */
	private void save_actionPerformed(ActionEvent evt) {
	
		
		try {
			saveLogin_actionPerformed(null);  //always saves
		} catch (NullPointerException e) {
		}
		
				
		String currentUserString = this.currentUser.toWXMLString();
		if (this.useEncryption.isSelected()) {
			String password = JOptionPane.showInputDialog(MultiLingual.translateWord(pname, "Please enter password") + ".");
			if (password == null)
				return;
			if (password.equals(""))
				return;
			String password2 = JOptionPane.showInputDialog(MultiLingual.translateWord(pname, "Please re-enter password") + ".");
			if (password2 == null)
				return;
			
			if (!password.equals(password2)) {
				JOptionPane.showMessageDialog(null, MultiLingual.translateWord(pname, "Password mis-match.  FILE NOT SAVED."), "Passwords Don't Match!", JOptionPane.OK_OPTION);
				return;
			}
			
			String key = OneTimeNOCList.generateRandomKey(currentUserString, password, 0);
			
			BufferedReader brPlain = new BufferedReader(new StringReader(currentUserString));
			BufferedReader brKey = new BufferedReader(new StringReader(key));
			StringWriter encryptedText = new StringWriter ();
			BufferedWriter bwencrypted = new BufferedWriter(encryptedText);
			try {
				OneTimeNOCList.encrypt(brPlain, brKey, bwencrypted);
			} catch (IOException e) {
				JOptionPane.showMessageDialog(null, MultiLingual.translateWord(pname, "Encryption error, try disabling encryption."), "Error!", JOptionPane.OK_OPTION);
			}
			
			currentUserString = encryptedText.toString();
		}
		
		TSFS.TSFSFile toWrite = new TSFS.TSFSFile (this.currentUserFileName, currentUserString);
		
		try {
			toWrite.writeFile();
			System.out.println("File: " + currentUserFileName + " has been saved!");
		} catch (IOException e) {
			System.out.println("Couldn't save file.");
		}
		
		saveNeeded = false;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);
		
	}
	

	/**
	 * Writes the ini file to disk.
	 *
	 */
	private void saveIni () {
		TSFS.TSFSFile toWrite = new TSFS.TSFSFile (Copernicus.INI_FILE_NAME, this.mainIni.toString());
		try {
			toWrite.writeFile();
		} catch (IOException e) {
			System.out.println("Couldn't save file.");
		}
	}
	
	/**
	 * Adds a new user to the system, and writes it to disk.
	 *
	 */
	private void addUserFile () {
		String filenameToAdd = null;
		filenameToAdd = TSFS.fileChooser(this, "Open User File", wxmlFilter, lastLoadedFile, null);
		
		if (filenameToAdd == null)
			return;
		lastLoadedFile = filenameToAdd;
		
		if (filenameToAdd == null || filenameToAdd.equals("")) {
			return;
		}
		
		boolean exists = false;
		for (int i = 0; i < users.size(); i++) {
			if ( ((String) users.get(i)).equals(filenameToAdd))
				exists = true;
		}
		
		if (!exists) {
			logout();
			lastUser++;
			users.add(filenameToAdd);
			
			
			loadUser(filenameToAdd);
			
			populateUsersList();
			try {
			users_name.put(currentUser.getSubSection("dpass").getAttributeValue("username"), filenameToAdd);
			users_filename.put(filenameToAdd, currentUser.getSubSection("dpass").getAttributeValue("username"));
			((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).setSubSectionValue("user_" + filenameToAdd +"_name",  currentUser.getSubSection("dpass").getAttributeValue("username"));
			} catch (Exception e) {
			}
			
//			users_name.put(currentUser.getSubSection("dpass").getAttributeValue("username"), filenameToAdd);
//			users_filename.put(filenameToAdd, currentUser.getSubSection("dpass").getAttributeValue("username"));			

			
			Ini.Key newKey = new Ini.Key("user" + lastUser, filenameToAdd, "");
//			Ini.Key newKey2 = new Ini.Key("user_" + filenameToAdd +"_name", currentUser.getSubSection("dpass").getAttributeValue("username"), "");
			mainIni.getIniSectionData().addSubSection(newKey);
//			mainIni.getIniSectionData().addSubSection(newKey2);
			saveIni();
		
		} else {
			JOptionPane.showMessageDialog(null, MultiLingual.translateWord(pname, "User exits, proceeding to load."), "User Exits.", JOptionPane.OK_OPTION);
			loadUser(filenameToAdd);
		}
	}
	
	
	/**
	 * Creates a new user.
	 *
	 */
	private void newUser () {
		logout();
		lastUser++;
		String newFileName = baseDirectory + "user" + this.lastUser + ".wxml";
		
		TSFS.TSFSFile newUser = new TSFS.TSFSFile(newFileName, Copernicus.firstUser(lastUser + ""));
		
		try {
			newUser.writeFile();
		} catch (IOException e) {
			System.out.println("Can't write new user file.");
		}
		users.add(newFileName);
		loadUser(newFileName);
		
		try {
			users_name.put(currentUser.getSubSection("dpass").getAttributeValue("username"), newFileName);
			users_filename.put(newFileName, currentUser.getSubSection("dpass").getAttributeValue("username"));
			((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).setSubSectionValue("user_" + newFileName +"_name",  currentUser.getSubSection("dpass").getAttributeValue("username"));
		} catch (Exception e) {
		}

		//users_name.put(currentUser.getSubSection("dpass").getAttributeValue("username"), newFileName);
		//users_filename.put(newFileName, currentUser.getSubSection("dpass").getAttributeValue("username"));
		
		Ini.Key newKey = new Ini.Key("user" + lastUser, newFileName, "");
		mainIni.getIniSectionData().addSubSection(newKey);
		//Ini.Key newKey2 = new Ini.Key("user_" + newFileName +"_name", currentUser.getSubSection("dpass").getAttributeValue("username"), "");		
		saveIni();
		
		populateUsersList();
		
		saveNeeded = true;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);
	}
	
	
	/**
	 * Removes the current user from the user list, and the ini file.  It doesn't erase it
	 * from the hard drive however.
	 */
	private void deleteCurrentUser () {
		try {
			((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).removeValue(this.currentUserFileName);
			users.remove(currentUserFileName);
			logout();
			saveIni();
			populateUsersList();		
		} catch (Ini.SubSectionNotFoundException e) {
		}
	}
	
	/**
	 * Creates a new site, requesting the name from the user.
	 *
	 *@param e The Action event.
	 */
	void addSite_actionPerformed(ActionEvent e) {
		String newSiteText =  JOptionPane.showInputDialog(MultiLingual.translateWord(pname, "Please enter new site name") + " (eg. www.hotmail.com).");
		if (newSiteText != null && !newSiteText.equals(""))
		createNewSite (newSiteText);
	}

	/**
	 * Creates a new site in memory.
	 *
	 *@param newSiteText the name of the new site.
	 */
	void createNewSite (String newSiteText) {
		this.siteNameText.setText(newSiteText);
		
		this.usernameText.setText("");
		this.passwordText.setText("");
		this.commentText.setText("");
		this.loginNameText.setText("Main Login");
		
		WXMLSection newSection = new WXMLSection("site", currentUser.getSubSection("dpass"));
		newSection.addAttribute(new WXMLSection.WXMLAttribute("name", this.siteNameText.getText(), true));
		
		//addLogin_actionPerformed(null);
		WXMLSection newLoginSection = new WXMLSection ("login", newSection);
		saveCurrentLogin(newLoginSection, true);
		
		newSection.addSubSection(newLoginSection);
		
		this.currentUser.getSubSection("dpass").addSubSection(newSection);
		
		populateSiteList();
		
		currentSite = newSection;
		lookupSite(false, true);
		
		this.lookupLogin.setEnabled(true);
		this.next.setEnabled(true);
		this.back.setEnabled(true);
		
		
		this.populateSiteList();
		this.populateLoginList();
		
		saveNeeded = true;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);
		
	}
	
	/**
	 * Updates the passed login with the text from the loginPanel.
	 *
	 *@param current The WXMLSection to be updated
	 *@param updateLoginName Whether or not to update the login name
	 */
	private void saveCurrentLogin (WXMLSection current, boolean updateLoginName) {
		saveNeeded = true;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);
		
		//currentLogin.getAttributeValue("userid")
		String loginNameToAdd = "";
		try {
			if (!updateLoginName || this.loginNameText.getText().equals(""))
				loginNameToAdd = currentLogin.getAttributeValue("userid");
			else
				loginNameToAdd = this.loginNameText.getText();
		
		} catch (NullPointerException e) {
			loginNameToAdd = this.loginNameText.getText();
		}
		
		current.purgeAll();
		
		current.setAttributeValue("userid", loginNameToAdd);
		current.setAttributeValue("username", this.usernameText.getText());
		current.setAttributeValue("pass", this.passwordText.getText());
		
		
		WXMLSection.WXMLAttribute commentAttr = new WXMLSection.WXMLAttribute ("comment", this.commentText.getText(), false);  
		current.addAttribute(commentAttr);
		
		//current.setAttributeValue("comment", this.commentText.getText()); //inline
		
		this.loginNameText.setText(current.getAttributeValue("userid"));
	}

	
	/**
	 * Saves the login to memory.
	 *
	 *@param e The Action event.
	 */
	void saveLogin_actionPerformed(ActionEvent evt) {
	
		try {
			if (currentLogin.getAttributeValue("userid").equals(this.loginNameText.getText())) {
				saveCurrentLogin(this.currentLogin, false);
		} else {
			saveCurrentLogin(this.currentLogin, false);
		}
			saveLogin.setEnabled(false);
		
			return;
		} catch (NullPointerException e) {
		}
		
		WXMLSection newLoginSection = new WXMLSection ("login", this.currentSite);
		saveCurrentLogin(newLoginSection, false);
		currentSite.addSubSection(newLoginSection);
		this.populateLoginList();
		saveNeeded = true;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);
		
		
		lookupLogin(loginNameText.getText());
		saveLogin.setEnabled(false);
		
		deleteLogin.setEnabled(true);
		
		menuRenameCurrentLogin.setEnabled(true);
		menuDeleteCurrentLogin.setEnabled(true);
	}
	
	
	/**
	 * Deletes the current login
	 *
	 *@param e The Action event.
	 */
	void deleteLogin_actionPerformed(ActionEvent e) {
		if (currentSite != null && currentLogin != null) {
			currentSite.removeSubSection(currentLogin);
			this.populateLoginList();
			currentLogin = null;
		} else {
			JOptionPane.showMessageDialog(null, MultiLingual.translateWord(pname, "Either the current login or current site has not yet been created."), "Nothing to delete!", JOptionPane.OK_OPTION);
		
		}
		saveNeeded = true;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);	
		saveLogin.setEnabled(true);
		
		menuDeleteCurrentLogin.setEnabled(false);
		menuRenameCurrentLogin.setEnabled(false);
		deleteLogin.setEnabled(false);
	
	}
	

	
	/**
	 * Changes the text of the lookup button to "Lookup" if that was pressed,
	 * and also runs the lookup when the Enter key is pressed.
	 *
	 *@param e The Action event.
	 */
	private void siteNameText_keyPressed (KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_ENTER) {
			checkLoginChanged(true);
                	lookupSite(true, false);
			siteNameText.requestFocus();
		} else {
	    		lookup.setText(MultiLingual.translateWord(pname, "Lookup"));
	    		searchNum = 0; //reset search position
		}
	    
	}
    

	/**
	 * Used to work out enabled state of login buttons
	 *
	 *@param e The Action event.
	 */
	void keypressed_action (KeyEvent e) {
	
		checkLoginChanged(false);
	}
	
	/**
	 * Prompts the user for a login name, and greates that login.
	 *
	 *@param e The Action event.
	 */
	void addLogin_actionPerformed(ActionEvent e) {
		
		// Prompt for save if nessesary
		checkLoginChanged(true);
		
		// Get new login name
		String newLoginText =  JOptionPane.showInputDialog(MultiLingual.translateWord(pname, "Please enter new login name") + "(eg. Main Login).");
		
		// Create new login
		if (newLoginText != null && !newLoginText.equals("")) {
			lookupLogin (newLoginText);
			
			saveNeeded = true;
			menuItemSave.setEnabled(saveNeeded);
			save.setEnabled(saveNeeded);
			deleteLogin.setEnabled(false);
			menuDeleteCurrentLogin.setEnabled(false);
			menuRenameCurrentLogin.setEnabled(false);
		}
	}
	
	/**
	 * Renames the current login.
	 *
	 *@param e The Action event.
	 */
	void menuRenameCurrentLogin_actionPerformed(ActionEvent e) {
		String newLoginText =  JOptionPane.showInputDialog(MultiLingual.translateWord(pname, "Please enter new login name") + ".");
		if (newLoginText != null && !newLoginText.equals("")) {
			
			currentLogin.setAttributeValue("userid", newLoginText);
			loginNameText.setText(newLoginText);
			
			saveNeeded = true;
			menuItemSave.setEnabled(saveNeeded);
			save.setEnabled(saveNeeded);
			
			saveCurrentLogin(currentLogin, true);
			populateLoginList();
		}
	}
	
	/**
	 * Renames the current site.
	 *
	 *@param e The Action event.
	 */
	void menuRenameCurrentSite_actionPerformed (ActionEvent e) {
		String newSiteText =  JOptionPane.showInputDialog(MultiLingual.translateWord(pname, "Please enter new site name."));
		if (newSiteText != null && !newSiteText.equals("")) {
			
			currentSite.setAttributeValue("name", newSiteText);
			siteNameLabel.setText("Site Name: " + newSiteText);
			siteNameText.setText(newSiteText);
			
			saveNeeded = true;
			menuItemSave.setEnabled(saveNeeded);
			save.setEnabled(saveNeeded);
			
			populateSiteList();
		}
	}

	/**
	 * Deletes the current site from memory.
	 *
	 *@param e The Action event.
	 */
	void deleteSite_actionPerformed(ActionEvent e) {
		if (currentSite != null) {
			this.siteNameText.setText(currentSite.getAttributeValue("name"));
			this.currentUser.getSubSection("dpass").removeSubSection(currentSite);
			currentSite = null;
		}
		this.populateSiteList();
		this.populateLoginList();
		
		saveNeeded = true;
		menuItemSave.setEnabled(saveNeeded);
		save.setEnabled(saveNeeded);
		
		this.usernameText.setText("");
		this.passwordText.setText("");
		this.commentText.setText("");
		this.loginNameText.setText("");
		this.usernameText.setEnabled(false);
		this.loginNameText.setEnabled(false);
		this.commentText.setEnabled(false);
		this.passwordText.setEnabled(false);
		this.deleteSite.setEnabled(false);
		this.menuRenameCurrentSite.setEnabled(false);
		this.menuDeleteCurrentSite.setEnabled(false);
		this.saveLogin.setEnabled(false);
		
		this.lookupLogin.setEnabled(false);
		this.next.setEnabled(false);
		this.back.setEnabled(false);
		
		this.deleteLogin.setEnabled(false);
		this.menuRenameCurrentLogin.setEnabled(false);
		this.menuDeleteCurrentLogin.setEnabled(false);
		this.addLogin.setEnabled(false);
		this.menuNewLogin.setEnabled(false);
		
		JOptionPane.showMessageDialog(null, MultiLingual.translateWord(pname, "Site Deleted"), MultiLingual.translateWord(pname, "Site Deleted"), JOptionPane.OK_OPTION + JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * Changes the default user entry in the ini file, and saves it.
	 */
	private void setCurrentAsDefault () {
		try {
			if (!this.logout.isEnabled()) {
				((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).setSubSectionValue("startingUser", "");
			} else {
				((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).setSubSectionValue("startingUser", this.currentUserFileName);
			}
			
			saveIni();
		} catch (Ini.SubSectionNotFoundException e) {
		}
	}
	
	/**
	 * Changes the language entry in the ini file, and saves it.
	 */
	private void setLanguageAsDefault () {
		try {
			((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).setSubSectionValue("language", MultiLingual.getCurrentLanguageCode() + "");
			saveIni();
		} catch (Ini.SubSectionNotFoundException e) {
		}
	}
	
	
	/**
	 * Opens the Save File as dialog box then saves the selected user into the specified filename.
	 *
	 *@param e The associated ActionEvent (can be null)
	 */
	void saveUserFileAs_actionPerformed(ActionEvent e) {
		try {
			String filename = null;
			filename = TSFS.fileChooser(this, "Save User File As", wxmlFilter, lastLoadedFile, "wxml");
			if (filename == null)
				return;
			lastLoadedFile = filename;
			
			//String filename = getFile("Save File As",FileDialog.SAVE, "*.wxml", baseDirectory);
			String old = this.currentUserFileName;
			this.currentUserFileName = filename;
			this.save_actionPerformed(null);
			
			String key = ((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).getSubSectionKey(old);
			((Ini.Section) mainIni.getIniSectionData().getSubSection("MAIN_SETTINGS")).setSubSectionValue(key, filename);
			
			users.remove(old);
			users.add(currentUserFileName);
			saveIni();
			populateUsersList();
		
		} catch (NullPointerException ex) {
		} catch (Ini.SubSectionNotFoundException ex) {		
		}
	}

	/**
	 * Code that is executed when the next and back buttons are pressed.
	 */
	public void navigateLogin (int direction) {
		try {
		        checkLoginChanged(true);

			WXMLSection [] logins =  getOrdedLogins();
			
			int cnt = 0;
			for (int i = 0; i < logins.length; i++) {
				
				if (logins[i] == currentLogin) {
					cnt = i;
					break;
				}
			}
			
			cnt += direction;
			
			if (cnt < 0)
				cnt = logins.length-1;
			if (cnt > logins.length -1)
				cnt = 0;
			if (cnt < 0)
				cnt = 0;
			
			lookupLogin( logins[cnt].getAttributeValue("userid"));
		} catch (Exception e) {
		}
	
	}


	/**
	 * Shows the help dialog box.
	 */
	private void showHelp () {
		String help = "Please refer to the accompanying help documentation.\n" +
			"Under Windows, one can access this from the start menu by\n" +
			" clicking on the help icon in the Copernicus folder.\n" +
			"Alternatively, it can be found in the docs sub-folder\n" +
			" of the directory where you installed Copernicus.\n" +
			"Some last minute information and notes can be found in the\n" +
			" accompanying readme.txt file.\n" +
			"Visit www.TankSoftware.com for more help and new versions.";
		JOptionPane.showMessageDialog(null, help, "Help.", JOptionPane.OK_OPTION + JOptionPane.INFORMATION_MESSAGE);
	}


	/**
	 * Changes the current lannguage.  The language menu is updated, the entire GUI is reloaded
	 * and the change is saved to disk.
	 *
	 *@param newLanguage The new language to be changed to.  See com.tanksoftware.io.MultiLingual for the possible values. 
	 */
	private void changeLanguage (String newLanguage) {
		updateLanguageTexts(newLanguage);
		setLanguageAsDefault();
		
		if (menuLanguages == null)
			return;
			
		JRadioButtonMenuItem [] menuLanguagesItems = new JRadioButtonMenuItem [menuLanguages.getItemCount()];
		
		for (int i = 0; i < menuLanguages.getItemCount(); i++) {
			//menuLanguagesItems[i] = (JRadioButtonMenuItem) menuLanguages.getItem(i); 
			//menuLanguagesItems[i].setSelected(false);
			
			if (((JRadioButtonMenuItem) menuLanguages.getItem(i)).getText().equals(newLanguage)) {
				((JRadioButtonMenuItem) menuLanguages.getItem(i)).setSelected(true);
			} else {
				((JRadioButtonMenuItem) menuLanguages.getItem(i)).setSelected(false);
			}
		}
		//menuLanguagesItems[newLanguage].setSelected(true);
		
		//updateLanguageTexts(newLanguage);
		//setLanguageAsDefault();
	}

	/**
	 * Reloads all of the text of the GUI, including menus, labels etc.
	 *
	 *@param newLanguage The new language to show.
	 */
	private void updateLanguageTexts (String newLanguage) {
	
		MultiLingual.setCurrentLanguageCode(newLanguage);
		//JLabel welcomeWagonLabel = new JLabel(MultiLingual.translateWord(pname, "Welcome"));
		loginLabel.setText(MultiLingual.translateWord(pname, "Current Login") + ":");
		usernameLabel.setText(MultiLingual.translateWord(pname, "username") + ":");
		passwordLabel.setText(MultiLingual.translateWord(pname, "password") + ":");
		commentLabel.setText(MultiLingual.translateWord(pname, "Comment") + ":");
		menuUsers.setText(MultiLingual.translateWord(pname, "Open User"));
		menuSites.setText(MultiLingual.translateWord(pname, "Sites"));
		menuLogins.setText(MultiLingual.translateWord(pname, "Logins"));
		
		menuCreateNewUser.setText(MultiLingual.translateWord(pname, "Create New User"));
		menuLogout.setText(MultiLingual.translateWord(pname, "Logout"));
		menuDeleteCurrentUser.setText(MultiLingual.translateWord(pname, "Delete Current User"));
		menuExit.setText(MultiLingual.translateWord(pname, "Exit"));
		menuSetAsDefault.setText(MultiLingual.translateWord(pname, "Set user as default"));
		
		menuExit.setText(MultiLingual.translateWord(pname, "Exit"));
		menuHelp.setText(MultiLingual.translateWord(pname, "Help"));
		menuHelp.setText(MultiLingual.translateWord(pname, "Help"));
		menuItemAbout.setText(MultiLingual.translateWord(pname, "About"));
		menuItemSave.setText(MultiLingual.translateWord(pname, "Save"));
		menuSaveAs.setText(MultiLingual.translateWord(pname, "Save User File As"));
		menuAddUserFile.setText(MultiLingual.translateWord(pname, "Open Existing User"));
		menuChangeName.setText(MultiLingual.translateWord(pname, "Change Display Name"));
		editUsername.setText(MultiLingual.translateWord(pname, "Change Display Name"));
		siteNameLabel.setText(MultiLingual.translateWord(pname, "Site Name") + ":");
		lookup.setText(MultiLingual.translateWord(pname, "Lookup"));
		addSite.setText(MultiLingual.translateWord(pname, "Create New Site"));
		lookupLogin.setText(MultiLingual.translateWord(pname, "Lookup Login"));
		exit.setText(MultiLingual.translateWord(pname, "Exit"));
		logout.setText(MultiLingual.translateWord(pname, "Logout"));
		save.setText(MultiLingual.translateWord(pname, "Save File"));
		saveLogin.setText(MultiLingual.translateWord(pname, "Save Login"));
		deleteLogin.setText(MultiLingual.translateWord(pname, "Delete Login"));
		addLogin.setText(MultiLingual.translateWord(pname, "New Login"));
		deleteSite.setText(MultiLingual.translateWord(pname, "Delete Site"));
		useEncryption.setText(MultiLingual.translateWord(pname, "Use Encryption"));
		menuCreateNewUser.setText(MultiLingual.translateWord(pname, "Create New User"));
		menuFile.setText(MultiLingual.translateWord(pname, "File"));
		menuItemHelp.setText(MultiLingual.translateWord(pname, "Help"));
		
		
		menuNewSite.setText(MultiLingual.translateWord(pname, "New Site"));
		menuRenameCurrentSite.setText(MultiLingual.translateWord(pname, "Rename Site"));
		menuDeleteCurrentSite.setText(MultiLingual.translateWord(pname, "Delete Site"));
		menuNewLogin.setText(MultiLingual.translateWord(pname, "New Login"));
		menuRenameCurrentLogin.setText(MultiLingual.translateWord(pname, "Rename Login"));
		menuDeleteCurrentLogin.setText(MultiLingual.translateWord(pname, "Delete Login"));
		
		menuEdit.setText(MultiLingual.translateWord(pname, "Edit"));

		// Position objects in the centre of the frame
		try {
			Dimension d = sitePanel.getPreferredSize();
			tabConsp1.setWidth(d.width + 15);
			tabConsp1.setX(0);
			
			Dimension d2 = loginPanel.getPreferredSize();
			tabConsp2.setWidth(d2.width + 15);
			tabConsp2.setX(0);
			
			Dimension d3 = editUsername.getPreferredSize();
			editUsernameXYConstraints.setX(0);
			
			pack();
			
			int fw = (int) ((double) (getWidth() - (d.width+15)) / 2.0);
			if (fw < 0)
				fw = 0;
			tabConsp1.setX(fw);
	
			
			fw = (int) ((double) (getWidth() - (d2.width+15)) / 2.0);
			if (fw < 0)
				fw = 0;
			tabConsp2.setX(fw);
	
			fw = (int) ((double) (getWidth() - (d3.width)) / 2.0);
			if (fw < 0)
				fw = 0;
			editUsernameXYConstraints.setX(fw);
			
		} catch (NullPointerException e) {
		}
	
		showWelcome();
		pack();
		setSize(getWidth() + 10, getHeight());
		// Add refresh code
		
		int fw = (int) ((double) (Toolkit.getDefaultToolkit().getScreenSize().width - getWidth()) / 2.0);
		int fh = (int) ((double) (Toolkit.getDefaultToolkit().getScreenSize().height - getHeight()) / 2.0);
		if (fw < 0)
			fw = 0;
		if (fh < 0)
			fh = 0;
		setLocation(fw,fh);
		
	}
	
}
