package com.tanksoftware.apps.morsecoder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import com.tanksoftware.io.*;

import com.tanksoftware.util.WillTank;
import com.tanksoftware.protocol.MorseCode;

/**
 * The main classes for the running of the application MorseCoder.
 *
 * MorseCodes.java
 *
 * Sets a GUI to be used for translation to and from morse code using the MorseCode class
 *
 *@author William Denniss
 *Version 19 June 2001
 */

public class MorseCoder extends JFrame{

	public static final SunFileFilter textFilter = new SunFileFilter("txt", "Text Files");

    String lastLoadedFile = "";


	// Program information
	public static final String PROGRAM_NAME = "Morse Coder";
	public static final String VERSION_NUMBER = "1.1";
	public static final String DATE_CREATED = "18 June 2001";
	public static final String DATE_UPDATED = "19 June 2001";
	public static final String AUTHOR = "William Denniss";

	public static WillTank willTankPanel = new WillTank(PROGRAM_NAME, VERSION_NUMBER, DATE_UPDATED);


	JPanel morsePanel;
	JTextArea morseText;
	String old;

	boolean firstFileLoad = true;
	String currentDirectory;
	String currentFilename;
	JMenuItem saveFileMI;


	//main method
	public static void main(final String[] args) {
              try {
              UIManager.setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
          } catch (Exception e) {
          }

  		willTankPanel.differWidth(+0);
		willTankPanel.startDialog();

		/*
		 * Try to execute the command line version of this program
		 */
		try {
			if (args[0] != null) {

				//Read in the input file
				String inputfile = "";
				String outputfile = "";
				String outputfilename = "";
				try {
					BufferedReader br = new BufferedReader(new FileReader(args[1]));

					String line = br.readLine();
					while (line != null) {
						inputfile += line;
						line = br.readLine();
						if (line != null)
							inputfile += "\n";
					}

					br.close();
				} catch (IOException e) {
					System.out.println("Error reading file " + args[1]);
					throw new ArrayIndexOutOfBoundsException ();
				}

				// Get outputfile name
				if (args.length == 3)
					outputfilename = args[2];
				else
					outputfilename = args[1];

				// Convert to desired output
				if (args[0].equalsIgnoreCase("-tomorse")) {
					outputfile = MorseCode.stringToMorse(inputfile);

				} else if (args[0].equalsIgnoreCase("-toascii")) {
					outputfile = MorseCode.stringFromMorse(inputfile);

				} else {
					System.out.println("Usage: java MorseCoder <mode> <sourcefile> [destinationfile]");
					throw new ArrayIndexOutOfBoundsException ();
				}

				// Write to output file
				try {
					PrintWriter pw = new PrintWriter(new FileWriter(outputfilename));

					pw.print(outputfile);
					pw.flush();
					pw.close();
				} catch (IOException e) {
					System.out.println("Error writing to file file " + outputfilename);
					throw new ArrayIndexOutOfBoundsException ();
				}

				System.out.println("File Processes Complete.");
				System.exit(0);
			}

		// If the command line option failed, open the GUI
		} catch (ArrayIndexOutOfBoundsException e) {
			final MorseCoder morse = new MorseCoder();
			morse.setBounds(50,50,450,491);
			morse.setTitle("Morse Coder");
			morse.pack();
			morse.setVisible(true);
			morse.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		}
	}


	/* Set up the GUI
	 */
	public MorseCoder () {

        String lastLoadedFile = TSFS.getBaseDirectory();

		morsePanel = new JPanel();


		JButton conv = new JButton("To Morse");
        conv.setIcon(new ImageIcon ("data/images/rightarrow.png"));

		conv.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				toMorse();
			}
		});

		JButton convB = new JButton("From Morse");
        convB.setIcon(new ImageIcon ("data/images/leftarrow.png"));

		convB.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				fromMorse();
			}
		});



		JPanel buttonPanel = new JPanel ();
		
		buttonPanel.add(conv);
		buttonPanel.add(convB);


		// create the text area
		morseText = new JTextArea("", 23, 52);

		// wrap the lines
		morseText.setLineWrap(true);

		// don't cut up words
		morseText.setWrapStyleWord(true);

		// Create a new JScrollPane
		JScrollPane scrollebar = new JScrollPane(morseText,ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,  ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		morsePanel.setLayout(new BorderLayout ());
		morsePanel.add(buttonPanel,BorderLayout.NORTH);
		morsePanel.add(scrollebar,BorderLayout.CENTER);


		morseText.setFont(new Font("Courier New", Font.PLAIN, 14));
//		morseText.setBounds(5,10,50,50);



		getContentPane().add(morsePanel);



		// name the menus
		JMenu file = new JMenu("File");
		JMenu edit = new JMenu("Edit");
		JMenu aboutm = new JMenu("About");

		// name the menuItems
		JMenuItem newFileMI = new JMenuItem("New") ;
        newFileMI.setIcon(new ImageIcon ("data/images/new.png"));
		JMenuItem openFileMI = new JMenuItem("Open");
        openFileMI.setIcon(new ImageIcon ("data/images/open.png"));
		saveFileMI = new JMenuItem("Save");
        saveFileMI.setIcon(new ImageIcon ("data/images/save.png"));
		JMenuItem saveAsFileMI = new JMenuItem("Save as");
        saveAsFileMI.setIcon(new ImageIcon ("data/images/blank.png"));
		JMenuItem exitMI = new JMenuItem("Exit");
        exitMI.setIcon(new ImageIcon ("data/images/blank.png"));
		JMenuItem aboutMI = new JMenuItem("About");
        aboutMI.setIcon(new ImageIcon ("data/images/tsicon.png"));

		JMenuItem undoMI = new JMenuItem("Undo");
        undoMI.setIcon(new ImageIcon ("data/images/undo.png"));
		JMenuItem toMorseMI = new JMenuItem("To Morse");
        toMorseMI.setIcon(new ImageIcon ("data/images/rightarrow.png"));
		JMenuItem fromMorseMI = new JMenuItem("From Morse");
        fromMorseMI.setIcon(new ImageIcon ("data/images/leftarrow.png"));



		saveFileMI.setEnabled(false);

		// add the menuItems to their menus
		file.add(newFileMI);
		file.add(openFileMI);
		file.add(saveFileMI);
		file.add(saveAsFileMI);
		file.addSeparator();
		file.add(exitMI);

		edit.add(undoMI);
		edit.addSeparator();
		edit.add(toMorseMI);
		edit.add(fromMorseMI);

		aboutm.add(aboutMI);

		//Create the menu bar
		JMenuBar menuBar = new JMenuBar();
		menuBar.add(file);
		menuBar.add(edit);
		menuBar.add(aboutm);
		//Add the menubar to the frame
		setJMenuBar(menuBar);




		repaint();


		//Add action listeners

		newFileMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				clear();
			}
		});

		aboutMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				willTankPanel.showDialog();
			}
		});

		openFileMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				load();
			}
		});


		saveFileMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				save();
			}
		});

		saveAsFileMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				saveAs();
			}
		});

		toMorseMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				toMorse();
			}
		});


		fromMorseMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				fromMorse();
			}
		});

		undoMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				undo();
			}
		});


		exitMI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				System.exit(0);
			}
		});

		// ACTION LISTENERS
	}

	public void undo () {
		String tmpOld = morseText.getText();
		morseText.setText(old);
		old = tmpOld;
	}

	/* Convert the JTextArea from morse code
	 */
	public void fromMorse () {

		String text = morseText.getText();
		old = text;
		String newText = MorseCode.stringFromMorse(text);
		morseText.setText(newText);

	}

	/* Clear the JTextArea
	 */
	public void clear () {
		old = morseText.getText();
		morseText.setText("");

	}

	/* Convert the JTextArea to morse code
	 */
	public void toMorse () {


		String text = morseText.getText();
		old = text;

		String newText = MorseCode.stringToMorse(text);

		morseText.setText(newText);
	}


	/* Save the JTextArea to a file
	 */
	public void saveFile (String filename) {
		try {
			PrintWriter pw = new PrintWriter(new FileWriter(filename));

			pw.print(morseText.getText());
			pw.close();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "There was an IO error writing to " + filename, "IO Error", JOptionPane.ERROR_MESSAGE );
		}


	}

	/* Save the current file
	 */
	public void save () {
		saveFile(currentFilename);
	}

	/* Save the JTextArea to the FileDialog file
	 */
	public void saveAs () {
		saveFileMI.setEnabled(true);

		String filename;
        /*
		FileDialog file = new FileDialog(this, "Save File", FileDialog.LOAD);
		file.setMode(FileDialog.SAVE);
		// sets the directory to the current one
		if (firstFileLoad) {
		// If it is the first time the dialog is loaded, load the working directory
			file.setDirectory(System.getProperty("user.dir"));
		} else {
			file.setDirectory(currentDirectory);
		}
		file.setFile("*.txt"); //sets the defult type to *.maze
		file.show(); //shows dialog box
        */

        filename = TSFS.fileChooser(this, "Save", textFilter, lastLoadedFile, "txt");
        if (filename == null)
            return;
        lastLoadedFile = filename;

        /*
		//This code will only execute if a file was selected (cancell was not pressed)
		if (file.getFile() != null && file.getFile() != "") {

			filename = file.getDirectory() + file.getFile();// + ".txt";
			if (!filename.substring(filename.length()-4).equals(".txt")) filename = filename + ".txt";
			currentDirectory = file.getDirectory(); //reset the loadup directory
			firstFileLoad = false; //the FileDialog has been loaded	successfully

		} else {

			return;

		}
        */
		currentFilename = filename;
		saveFile(filename);


	}


	/* Load the JTextArea from the FileDialog file
	 */
	public void load () {


		String filename;
		/*FileDialog file = new FileDialog(this, "Open File", FileDialog.LOAD);
		file.setMode(FileDialog.LOAD);

		// sets the directory to the current one
		if (firstFileLoad) {
		// If it is the first time the dialog is loaded, load the working directory
			file.setDirectory(System.getProperty("user.dir"));
		} else {
			file.setDirectory(currentDirectory);
		}
		file.setFile("*.txt"); //sets the defult type to *.maze
		file.show(); //shows dialog box

		//This code will only execute if a file was selected (cancell was not pressed)
		if (file.getFile() != null && file.getFile() != "") {
			filename = file.getDirectory() + file.getFile();
			currentDirectory = file.getDirectory(); //reset the loadup directory
			firstFileLoad = false; //the FileDialog has been loaded	successfully

		} else {

			return;

		}
        */
        filename = TSFS.fileChooser(this, "Open", textFilter, lastLoadedFile, null);
        if (filename == null)
            return;
        lastLoadedFile = filename;


		currentFilename = filename;
		saveFileMI.setEnabled(true);


		try {

			BufferedReader br;
			//Title and Author
			br = new BufferedReader(new FileReader(filename));
			String line = br.readLine();
			boolean first = true;
			while (line != null) {
				if (first) {
					morseText.setText(line);
					first = false;
				} else {
					morseText.setText(morseText.getText() + "\n" + line);
				}
				line = br.readLine();
			}

			br.close();
		} catch (IOException e) {

			JOptionPane.showMessageDialog(null, "There was an IO error reading from " + filename, "IO Error", JOptionPane.ERROR_MESSAGE );
		}


	}


}
