package com.tanksoftware.apps.numbertranslator;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

import com.tanksoftware.protocol.NumberTranslate;
import com.tanksoftware.util.WillTank;
import com.tanksoftware.util.RedirectedFrame;



/**
 * The main classes for the running of the application NumberTranslator.
 *
 * NumberTranslator.java
 * NumberTranslator was created by William Denniss in March 2001.  The original DOS version was created in 1998
 * This class does the following:
 * - Draws all of the GUI components,
 * - Stars a new Thread when the Start button is pressed to output the translation
 * William Denniss has asserted his right under the Copyright, Designs and Patents Act 1988 to be identified as the author of this work.
 * Why this program was developed:
 * Partially as an personal exercise, however the original idea came when I was watching a "current affair" type program.
 * featured on this program was a story about a little old lady who sat at her typewriter and typed out numbers in words in sequence.
 * I thought to my self "I think I could do a computer program to do that" and here it is.
 */


public class NumberTranslator extends JFrame {

	// Program information
	public static final String PROGRAM_NAME = "NumberTranslator";
	public static final String VERSION_NUMBER = "2.4";
	public static final String DATE_CREATED = "March 2001";
	public static final String DATE_UPDATED = "13 April 2001";
	public static final String AUTHOR = "William Denniss";


	// GUI variables
	private JPanel labelPanel;
	private JPanel inputPanel;
	private JPanel displayPanel;
	private JPanel buttonPanel;
	private JPanel backPanel;
	private JPanel optionsPanel;
	private JPanel checkBoxPanel;
	private JPanel radioPanel;
	private JTextField minimumInput;
	private JTextField maximumInput;
	private JButton filenameButton;
	private JButton startButton;
	private JButton stopButton;

  	// radio buttons and check boxes
	private String translateAction = "single";
	private JCheckBox outputToScreenButton;
	private JCheckBox outputToFileButton;
	private JRadioButton singleButton;
	private JRadioButton continuousButton;
	private JRadioButton randomButton;

	// labels
	private JLabel minLabel = new JLabel("Minimum: ");
	private JLabel maxLabel = new JLabel("Maximum: ");
	private JLabel optLabel = new JLabel("Translation: ");



	// File Dialog variables
	private boolean firstFileLoad = true;
	private String currentDirectory;
	private String filename = "NumberTranslator.txt"; //set default filename

	// translation variables
	private boolean continueLoop = true;
	private boolean currentlyTranslating = false;
	private boolean paused = false;
	private boolean outputToFile = true;
	private boolean outputToScreen = true;

	private String translatedNumber;

	private long inputMinimum = 0;
	private long inputMaximum = 0;
	private long counter = 0;

	// Load the Redirect Frame.  This frame will divert all System.out.println(); messages from the command line, to this frame
//	RedirectedFrame outputFrame = new RedirectedFrame();
	RedirectedFrame outputFrame = new RedirectedFrame(false, false, null, 700, 600, JFrame.DO_NOTHING_ON_CLOSE);
	private static WillTank willTankPanel = new WillTank(PROGRAM_NAME, VERSION_NUMBER, DATE_UPDATED);



	//main method
	public static void main(final String[] args) {
		// Create a new Frame
		final NumberTranslator translator = new NumberTranslator();

		// Set the Program to terminate when this frame is closed
		translator.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Set the icon file
		Toolkit tk = Toolkit.getDefaultToolkit();
		Image im = tk.getImage("data/NumberTranslator/numberIcon.png");
		translator.setIconImage(im);

		willTankPanel.startDialog();

	}


	/** NumberTranslator **
	This Method is the constructor and creates a new Frame, with text boxes, buttons, radio buttons and check boxes
	*/
	public NumberTranslator () {
        try {
        UIManager.setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
        } catch (Exception e) {
          e.printStackTrace(System.out);
        }

		//labels
		labelPanel = new JPanel();
		labelPanel.setLayout(new GridLayout(2,1));
		labelPanel.add(minLabel);
		labelPanel.add(maxLabel);

		//text boxes
		inputPanel = new JPanel();
		inputPanel.setLayout (new GridLayout(2,1));
		minimumInput = new JTextField(15);
		inputPanel.add(minimumInput);
		maximumInput = new JTextField(15);
		maximumInput.setEditable(true);
		maximumInput.setColumns( 9);
		inputPanel.add(maximumInput);

		//add labels and text boxes
		displayPanel = new JPanel();
		displayPanel.setLayout(new BorderLayout());
		displayPanel.add(labelPanel,BorderLayout.WEST);
		displayPanel.add(inputPanel, BorderLayout.CENTER);


		// RADIO BUTTONS

	    // Create the radio buttons.
	    singleButton = new JRadioButton("single");
	    //singleButton.setMnemonic(KeyEvent.VK_S); // This line is not required, it would underline the s
	    singleButton.setActionCommand("single");

		singleButton.setSelected(true); //This radio button will be the first selected

		continuousButton = new JRadioButton("continuous");
		continuousButton.setActionCommand("continuous");

	    randomButton = new JRadioButton("random");
	    randomButton.setActionCommand("random");

	    // Group the radio buttons.
	    ButtonGroup group = new ButtonGroup();
	    group.add(singleButton);
	    group.add(continuousButton);
	    group.add(randomButton);

	    // Register a listener for the radio buttons. The RadioListener class is an inner class and is declared below
	    RadioListener radioListener = new RadioListener();
	    singleButton.addActionListener(radioListener);
	    continuousButton.addActionListener(radioListener);
	    randomButton.addActionListener(radioListener);

	    // Add the radio boxes to a Panel
	    radioPanel = new JPanel();
	    radioPanel.add(optLabel);
	    radioPanel.add(singleButton);
	    radioPanel.add(continuousButton);
	    radioPanel.add(randomButton);


	    // CHECK BOXES

	    // Create check boxes
		outputToScreenButton = new JCheckBox("Show Output");
		//outputToScreenButton.setMnemonic(KeyEvent.VK_C);
		outputToScreenButton.setSelected(true);

		outputToFileButton = new JCheckBox("Output to File");
		outputToFileButton.setSelected(true);


		// Register a listener for the check boxes. The CheckBoxListener class is an inner class and is declared below
		CheckBoxListener checkBoxListener = new CheckBoxListener();
		outputToScreenButton.addItemListener(checkBoxListener);
		outputToFileButton.addItemListener(checkBoxListener);

		// Add the check boxes to a Panel
 		checkBoxPanel = new JPanel();
		checkBoxPanel.add(outputToScreenButton);
	    checkBoxPanel.add(outputToFileButton);
		filenameButton = new JButton("Select File", new ImageIcon ("data/images/open.png"));
		checkBoxPanel.add(filenameButton);

		// Merge all of the Check Boxes and Radio Buttons to a single Panel
		optionsPanel = new JPanel();
		optionsPanel.setLayout(new BorderLayout());
		optionsPanel.add(radioPanel, BorderLayout.NORTH);
		optionsPanel.add(checkBoxPanel, BorderLayout.CENTER);


		// Create the buttons
		startButton = new JButton("Start", new ImageIcon ("data/images/start.png"));
		stopButton = new JButton("Stop", new ImageIcon ("data/images/stop.png"));

		// Add the buttons, and a label to a single Panel
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new BorderLayout());
		buttonPanel.add(startButton, BorderLayout.WEST);
		buttonPanel.add(new JLabel("created by William Denniss", JLabel.CENTER), BorderLayout.CENTER);
		buttonPanel.add(stopButton, BorderLayout.EAST);


		// Merge ALL of the GUI components
		backPanel = new JPanel();
		backPanel.setLayout(new BorderLayout());
		backPanel.add(optionsPanel,BorderLayout.NORTH);
		backPanel.add(displayPanel,BorderLayout.CENTER);
		backPanel.add(buttonPanel,BorderLayout.SOUTH);

		// Add the merged Panel to the Frame
		getContentPane().add(backPanel);

		//add action listeners for buttons and textboxs
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				startTranslation();
			}
		});

		stopButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				stopTranslation();
			}
		});


		filenameButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				chooseFile();
			}
		});

		minimumInput.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				startTranslation();

			}
		});

		maximumInput.addActionListener(new ActionListener () {
			public void actionPerformed(ActionEvent e) {
				startTranslation();
			}
		});

		addKeyListener(new KeyAdapter( ) {
			public void keyPressed (KeyEvent e) {
				keyPressedEvent(e); //detect if the keyboard is pressed and if so, execute the showNextPiece method
			}
		});


		// Set the Frame options
		pack();
		setTitle("NumberTranslator");
		setLocation(300, 200); //set distance from top left corner of screen
		setVisible(true); //show the frame
		setResizable(false); //don't let the user resize the frame
	}


	/** keyPressedEvent **
	This Method is invoked by pressing a key on the keyboard.
	If the user presses F5, then the about box is displayed
	*/
	public void keyPressedEvent (KeyEvent e) {
		int keyCode = e.getKeyCode(); //extracts the keycode from the KeyEvent

		if (keyCode == KeyEvent.VK_F5) {

			willTankPanel.showDialog();

		}

	}



	/** startTranslation **
	This Method is invoked by clicking the Start button, or pressing "Enter" in a text box
	It creates a new thread of the Class (inner) TranslateThread, which does the translations.
	The purpose of starting a new thread, is so the GUI is still active, and the user can exit, pause the translation etc.
	*/
	private void startTranslation () {

		boolean proceed = true;
		boolean larger = false;

		// If the program is not currently Translation a number, start a new translation
		if (!currentlyTranslating) {

			// This is the maximum input that will be allowed
			Long maximumSize = new Long("999999999999999999");

			// Check to make sure the number is a "long"
			try {
				inputMinimum = Long.parseLong(minimumInput.getText());
				if (inputMinimum < 0 || maximumSize.compareTo(new Long(inputMinimum)) < 0)
					proceed = false;
			} catch (NumberFormatException e) {
				proceed = false;
			}

			try {
				if (!translateAction.equals("single")) {
					inputMaximum = Long.parseLong(maximumInput.getText());
					if (inputMaximum < 0 || maximumSize.compareTo(new Long(inputMinimum)) < 0)
						proceed = false;
					Long minimumLong = new Long(inputMinimum);
					if (minimumLong.compareTo(new Long(inputMaximum)) > 0)
						larger = true;

				}
			} catch (NumberFormatException e) {
				proceed = false;
			}


			// If the number isn't a long, show an error message
			if (!proceed) {
				JOptionPane.showMessageDialog(this, "You must enter a valid number\n" //the \n creates a new line
													+"This program supports positive 'long' numbers\n"
													+"which are whole numbers in the range\n"
													+"0 to 999,999,999,999,999,999", "Invalid Number!", JOptionPane.ERROR_MESSAGE);
			}

			if (larger) {
				JOptionPane.showMessageDialog(this, "The minimum number must be smaller than the maximum", "Invalid Number!", JOptionPane.ERROR_MESSAGE);
			}

			// If the number is a long, then start the translation
			if (proceed && !larger) {

				//the next statement would start a new translation thread every time it is run
				final Thread translator = new TranslateThread();
				translator.setPriority(Thread.MAX_PRIORITY); // give the thread the maximum priority (over the GUI)
				translator.start(); //this will invoke the run() method of translator (see below for the TranslateThread class)

				// Disable the Check Boxes, and change the buttons, only if it is not a "single" translation
				if (!translateAction.equals("single")) {
					currentlyTranslating = true;
					startButton.setText("Pause");
					singleButton.setEnabled(false);
					continuousButton.setEnabled(false);
		  			randomButton.setEnabled(false);
					minimumInput.setEnabled(false);
					maximumInput.setEnabled(false);
					minLabel.setEnabled(false);
					maxLabel.setEnabled(false);
					optLabel.setEnabled(false);
				}

			}
		// If the program is currently translating, pause the current translation
		} else {


			if (!translateAction.equals("single")) {
		 		continueLoop = false;
				currentlyTranslating = false;
				paused = true;
				startButton.setText("Resume");
			}
		}

	}

	/** stopTranslation **
	This method stops the translation, and enables the radio buttons
	The TranslateThread class will only run if continueLoop is true
	*/
	private void stopTranslation () {
 		continueLoop = false;
		currentlyTranslating = false;
		paused = false;
		startButton.setText("Start");
		singleButton.setEnabled(true);
		continuousButton.setEnabled(true);
		randomButton.setEnabled(true);
		minimumInput.setEnabled(true);
		maximumInput.setEnabled(true);
		minLabel.setEnabled(true);
		maxLabel.setEnabled(true);
		optLabel.setEnabled(true);


	}

	/** chooseFile **
	This method opens a File Dialog box, prompting the user to set a filename
	If this method isn't run, the default filename declared above is used
	*/
	private void chooseFile () {

		// Load the File Dialog Box
		FileDialog file =
        new FileDialog(this, "Select File to Append");//, 		      FileDialog.LOAD);

		// If the FileDialog has been opened successfully, set it to the directory when it was closed.  If not, then the current directory will be used (by default).
		if (firstFileLoad) {
			file.setDirectory(System.getProperty("user.dir"));
		} else {
	    	file.setDirectory(currentDirectory);
		}

		file.setFile("*.txt"); //sets the defult type to *.maze
		file.show(); //shows the dialog box

		//This code will only execute if a file was selected (cancel was not pressed)
		if (file.getFile() != null) {
			filename = file.getDirectory() + file.getFile(); // assign the file and directory to the filename String
			currentDirectory = file.getDirectory(); //reset the loadup directory
			firstFileLoad = false; //the FileDialog has been loaded	successfully

		}

	}


	/** RadioListener **
	This inner Class listens to the Radio Buttons
	When one is clicked the translationAction String is changed to the String contained in the action command
	eg. singleButton.setActionCommand("single"); when the Single radio button is clicked, translateAction would be "single"
	*/
	class RadioListener implements ActionListener {
  	public void actionPerformed(ActionEvent e) {
		// Assign the declared radio button command to the translateAction String
  		translateAction = e.getActionCommand();

	  }
  	}


	/** CheckBoxListener **
	This inner Class listens to the Check boxes
	When one is clicked the translationAction String is changed to the String contained in the action command
	eg. singleButton.setActionCommand("single"); when the Single radio button is clicked, translateAction would be "single"
	*/
	class CheckBoxListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {

			Object source = e.getItemSelectable();

			// If the outputToScreenButton checkbox was clicked:
			if (source == outputToScreenButton) {

				// Change the outputToScreen boolean to reflect the change
	  			if (outputToScreen) {
	  				outputToScreen = false;
	  				outputFrame.setVisible(false); // hide outputFrame
	  			} else {
					outputToScreen = true;
	   				outputFrame.setVisible(true);
					setVisible(true); //place the main frame in front of the output frame
	  			}

			// If the outputToFileButon checkbox was clicked:
			} else if (source == outputToFileButton) {

				if (outputToFile) {
	  				outputToFile = false;
	  			} else {
	  				outputToFile = true;
	  			}

			}

		   // possible use: if (e.getStateChange() == ItemEvent.DESELECTED)
		}
	}


	/** TranslateThread **
	This inner Class starts a new thread which does the number translation


	It is invoked by using the following code:
	Thread translator = new TranslateThread();
	translator.start();
	*/
	public class TranslateThread extends Thread {


		/** run **
		The run medhod overrides the default run method for this class.
		It is activated by the line start();
		This method contains the code to start a translation.
		*/
		public void run () {

			// Create a new object of the class Translate
			NumberTranslate words = new NumberTranslate();
			continueLoop = true;

			// Get the variables from the GUI
			if (!paused) {

				// Reset the variables
				inputMinimum = 0;
				inputMaximum = 0;
				counter = 0;

				// Try to get these variables from the text boxes
				try {
					inputMinimum = Long.parseLong(minimumInput.getText());
				} catch (NumberFormatException e) {
					inputMinimum = 0;	//an invalid number was entered.
				}

				try {
					inputMaximum = Long.parseLong(maximumInput.getText());
				} catch (NumberFormatException e) {
					inputMaximum = 0;	//an invalid number was entered.
				}

				counter = inputMinimum;

				if (translateAction.equals("continuous")) counter = inputMinimum - 1; //inputMinimum is reduced by one if it is continuous to compensate for it being increased before the first translation

			} else {
				paused = false;	// the translation has now been unpaused
			}


			while (continueLoop) {

				// If a single translation is required, run the loop only once
				if (translateAction.equals("single")) {
					continueLoop = false;

				// If a continuous translation is being preformed, increase the counter and test to see if the Maximum number has been reached
				} else if (translateAction.equals("continuous")) {
					counter++;
					if (counter == inputMaximum) continueLoop = false;

				// If the translation is random, set the counter to a random number, confined by the inputted minimum and maximum
				} else if (translateAction.equals("random")) {
					counter = (int)((inputMaximum-inputMinimum+1)*Math.random()+inputMinimum);
				}

				// Sends the number in counter to the Translate class which will return its translated number as a String
				translatedNumber = NumberTranslate.longTranslation(counter);//words.Translate(counter);

				// If outputToScreen is set to true, then ouput the String. This will actually go to the RedirectFrame, because that is redirecting all System.out.println() messages to itself
				if (outputToScreen) {
					System.out.println(translatedNumber);
				}

				// If outputToFile is set to true, append the selected file with the ouput
				if (outputToFile) {

					BufferedWriter bw = null;

					try {

						// Append to the file, the new line
						bw = new BufferedWriter(new FileWriter(filename, true));
						bw.write(translatedNumber); //adds a new line to the file containing a comment and the date.
						bw.newLine();
						bw.flush();

					} catch (IOException ioe) {
						// Print the error message to the system
						ioe.printStackTrace();

					// Always close the file
					} finally {
						if (bw != null) {
							try {
								bw.close();
							} catch (IOException ioe2) {
									//Ignore IOException
							}
						}
					}
				}
			}

			// If the while loop terminated for a reason other than it being paused, reset the variables
			if (!paused) stopTranslation();

		}
	}


}


