//WP2Object.java
package com.tanksoftware.graphics;

import java.awt.Image;
import java.awt.Graphics;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;


/**
* An image object that amongst others, can be moved, drawn and tested if it intersects another WP2Object.
 *
 *@author William Denniss
 *@created 21 Feb 2002
 *
 *@version 1.0, 21 Feb 2002
 *
 * Versions:
 * 1.0 - 21 Feb 2002: Started.
 * 1.1 - 28 Feb 2002: Bug in intercect fixed, and the intercect method was heavily optimised.
 *
 */



public abstract class WP2Object {


	protected int x;
	protected int y;
	protected int height;
	protected int width;
	//protected int intercectSteps = 5;

	protected Image picture;

	protected boolean alive = true;


	public int getWidth () { return width; }
	public int getHeight () { return height; }
	public int getX () { return x; }
	public int getY () { return y; }
	public double getArea () { return width * height; }

	/**
	 * boolean intercect (WP2Object secondObject)
	 *
	 * Determine wether or not this object intercects the passed one.
	 *
	 */
	public boolean intercect (WP2Object secondObject) {
		int i,j;

		// Optimisation: Only run if both objects are alive
		if (!isAlive() || !secondObject.isAlive())
			return false;

		// Optimisation: Intercect will only ever be performed on the smaller of the two objects (less iterations of the for loops)
		// The smaller the object, the smaller the area that is needed to test.
		if (secondObject.getArea() < this.getArea())
			return secondObject.intercect(this);

		// Optimisation: The mimimum number of iterations will be used, depending on the object.
		// The bigger the object, the less the iterations wil be needed.
		int xsteps = (int) (secondObject.getWidth() * 0.8);
		int ysteps = (int) (secondObject.getHeight() * 0.8);

		j = y;
		for (i = x; i < x + width; i += xsteps)
			if (secondObject.inside(i,j))
				return true;

		j = y+height;
		for (i = x; i < x + width; i += xsteps)
			if (secondObject.inside(i,j))
				return true;
		i = x;
		for (j = y; j < y + height; j += ysteps)
			if (secondObject.inside(i,j))
				return true;

		i = x + width;
		for (j = y; j < y + height; j += ysteps)
			if (secondObject.inside(i,j))
				return true;

		return false;

	}

	/**
	 * boolean inside (int x, int y)
	 *
	 * Determine wether or not a given point is inside this object.
	 *
	 */
	public boolean inside (int x2, int y2) {

		if (x2 >= this.x && y2 >= this.y && x2 <= this.x+width && y2 <= this.y + height)
			return true;
		else
			return false;

	}

	/**
	 * void draw (Graphics g)
	 *
	 * Draw this object to the Graphics controller
	 *
	 */
	public void draw (Graphics g) {
		if (alive)
			g.drawImage(picture, x, y, null);

	}
//	public void draw (Graphics g, JPanel p) {
//		g.drawImage(picture, x, y, p);
//
//	}

	public void moveAbsolute (int x, int y) {
		this.x = x	;
		this.y = y;
	}

	public void kill () {
		alive = false;
	}


	public boolean isAlive () {
		return alive;
	}

	/**
	 * Removes any dead items from the item list.
	 * Best run in between levels, while paused etc, after a certain limit is reached.
	 * The number of dead objects in any given level should not cause too many problems,
	 * unless this number is exceptionally high
	 *
	 */
	public static List cleanup (List toClean) {
		List cleaned = new ArrayList();
		for (int i = 0; i < toClean.size(); i++) {
			WP2Object current = (WP2Object) toClean.get(i);
			if (current.isAlive())
				cleaned.add(current);
		}

		return cleaned;
	}

	public abstract boolean isType (String type);

	public abstract WP2Object deepClone ();

	public abstract String toWXMLString ();
}