package com.tanksoftware.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.StringReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;

import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.*;



import java.util.List;
import java.util.ArrayList;

import java.net.URL;
import java.net.MalformedURLException;
import java.net.URLConnection;

/** TSFS
 *
 * The Tank Software File System is a comprehensive file management system.
 * It can be used both so that files are only ever read from the disk once,
 * and as a way to load one physical file as several logical ones. It can
 * also be used staticly mearly to obtain a file-specific BufferedReader
 * depending on the location of the file (ie. is it a URL).
 *
 *
 *@author William Denniss
 *
 * Versions:
 * 1.0 - 3 April 2002: TSFS Created
 *
 * William Denniss has asserted his right under the Australian Copyright, Designs and Patents Act 1988 to be identified as the author of this work.
 */

public class TSFS {

	//String baseDirectory;
	List files = new ArrayList();

	//public TSFS (String baseDirectory) {
	//	this.baseDirectory = baseDirectory;
	//}

	public BufferedReader loadFile (String filename) throws IOException{
		return loadFile(filename, false);

	}

	public List getFiles () {
		return files;
	}

	public BufferedReader loadFile (String filename, boolean reload) throws IOException {

		System.out.println("File System: Loading file: " + filename);

		if (!reload) {
			for (int i = 0; i < files.size(); i++) {
				TSFSFile current = (TSFSFile) files.get(i);
				if (current.filename.equals(filename)) {
					return new BufferedReader(new StringReader (current.fileContents));
				}
			}
		}

		System.out.println("File System: Reading file: " + filename);

		TSFSFile newFile = new TSFSFile (filename);
		files.add(newFile);
		return new BufferedReader(new StringReader (newFile.fileContents));


	}

	public void loadCollection (String file, String baseDirectory) throws IOException {


		TSFSFile collection = new TSFSFile(file);
		BufferedReader br = new BufferedReader(new StringReader(collection.fileContents));

		String line = br.readLine();
		while (line != null) {
			try {

				if (line.substring(0,3).equals("***")) {
					String currentFilename;
					String currentFileContents = "";
					currentFilename = baseDirectory + line.substring(3,line.length());
					line = br.readLine();
					//System.out.println("asdfadfadsf");
					//System.out.println(line);
					boolean continueParsing = true;
					while (continueParsing) {
						currentFileContents += line + "\n";
						line = br.readLine();
						//System.out.println(line);
						try {
							continueParsing = !line.substring(0,3).equals("***");
						} catch (StringIndexOutOfBoundsException e) {
						}
					}
					this.files.add(new TSFSFile(currentFilename, currentFileContents));
					line = br.readLine();

				} else {
					line = br.readLine();
				}
			} catch (StringIndexOutOfBoundsException e) {
				line = br.readLine();
			}
		}


	}


    public static String getBaseDirectory () {

        try {
     	    File f = new File("");
    		String baseDirectory = f.getAbsolutePath() + "/";
            return baseDirectory;
        } catch (Exception e) {
            //this.getCodeBase().toString();
            return "";
        }

    }

	public static String fileChooser (Component component, String title, FileFilter filter, String workingDir, String defaultExt) {

		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setCurrentDirectory(new File (workingDir ));
		try {
		    fileChooser.addChoosableFileFilter(filter);
		} catch (NullPointerException e) {
		}
		fileChooser.setDialogTitle(title);
		int result = fileChooser.showOpenDialog(component);
		String sep = System.getProperty("file.separator");

		if(result == JFileChooser.APPROVE_OPTION) {
            String addition = "";
            if (defaultExt != null && fileChooser.getSelectedFile().getName().lastIndexOf(".") == -1) {
                addition = "." + defaultExt;
            }
			workingDir = fileChooser.getCurrentDirectory().toString();
            String tmp = fileChooser.getCurrentDirectory().getAbsolutePath()+sep + fileChooser.getSelectedFile().getName() + addition;
            System.out.println(tmp);
			//tmp = fileChooser.getCurrentDirectory().toString() +sep + fileChooser.getSelectedFile().getName() + addition;
			return tmp;
		} else {
			return null;
		}

	}


    /**
     * A single file who has it's text data pre-loaded from the disk when it is created.
     */
	public static class TSFSFile {

		private String fileContents;
		private String filename;

		public TSFSFile (String filename, String fileContents) {
			this.fileContents = fileContents;
			this.filename = filename;
		}

		public String getFileContents () {
			return fileContents;
		}

		public String getFilename () {
			return filename;
		}

		public void writeFile () throws IOException {
			writeFile(filename);
		}

		public void setFileContents (String fileContents) {
			this.fileContents = fileContents;
		}

		public void writeFile (String filename) throws IOException {
			writeFile(filename, fileContents);
		}
		
		public static void writeFile (String filename, String fileContents) throws IOException {
			BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename), "UTF-8"));
			bw.write(fileContents);
			bw.close();
			
		}

		public static BufferedReader getFileSpecificBufferedReader (String filename) throws IOException {
			BufferedReader br = null;

			// Cater for applets, which will have http:// or file:/ in their paths.
			if (filename.lastIndexOf(":/") == -1) {
				br = new BufferedReader(new InputStreamReader( new FileInputStream(filename), "UTF-8"));
			} else {
				try {
					//Get the BufferedReader to read an URL
					URL url = new URL(filename);

					URLConnection conn = null;
					InputStreamReader data = null;
					conn = url.openConnection();
					conn.connect();
					br = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));

					//br = new BufferedReader(new FileReader(new File(new URI(filename))));
				} catch (MalformedURLException e) {
				}
			}

			return br;
		}

		public TSFSFile (String filename) throws IOException {
			this.filename = filename;

			BufferedReader br = TSFSFile.getFileSpecificBufferedReader(filename);

			if (br == null)
				throw new IOException ();

			fileContents = "";
			String line = br.readLine();
			while (line != null) {
				fileContents += line + "\n";
				line = br.readLine();
			}


		}


	}

    public class CancelException extends Exception {
    }
}